# -*- encoding: utf-8 -*-
import math
import numbers
import re
from abjad.tools.abctools.AbjadObject import AbjadObject


class Octave(AbjadObject):
    r'''An octave.

    ::

        >>> pitchtools.Octave(4)
        Octave(4)

    ::

        >>> pitchtools.Octave(",,")
        Octave(1)

    ::

        >>> pitchtools.Octave(NamedPitch("cs''"))
        Octave(5)

    ::

        >>> pitchtools.Octave(pitchtools.Octave(2))
        Octave(2)

    Returns octave.
    '''

    ### CLASS VARIABLES ###

    _octave_tick_regex_body = """
        (,+     # one or more commas for octaves below the bass clef
        |'+     # or one or more apostrophes for the octave of the treble clef
        |)      # or empty string for the octave of the bass clef
        """

    _octave_tick_regex = re.compile(
        '^{}$'.format(_octave_tick_regex_body),
        re.VERBOSE,
        )

    __slots__ = (
        '_octave_number',
        )

    ### INITIALIZER ###

    def __init__(self, octave_number=None):
        from abjad.tools import pitchtools
        expr = octave_number
        if isinstance(expr, numbers.Number):
            octave_number = int(expr)
        elif isinstance(expr, str):
            match = self._octave_tick_regex.match(expr)
            if match is None:
                message = 'can not instantiate octave: {!r}.'
                message = message.format(expr)
                raise Exception(message)
            group = match.group()
            if group == '':
                octave_number = 3
            elif group.startswith("'"):
                octave_number = 3 + len(group)
            else:
                octave_number = 3 - len(group)
        elif isinstance(expr, pitchtools.Pitch):
            octave_number = expr.octave_number
        elif isinstance(expr, type(self)):
            octave_number = expr.octave_number
        elif expr is None:
            octave_number = 4
        else:
            message = 'can not instantiate {}: {!r}.'
            message = message.format(type(self), expr)
            raise Exception(message)
        self._octave_number = octave_number

    ### SPECIAL METHODS ###

    def __eq__(self, other):
        r'''True if `other` is octave with same octave number. Otherwise False.

        ::

            >>> octave = pitchtools.Octave(4)
            >>> octave == pitchtools.Octave(4)
            True

        ::

            >>> octave == pitchtools.Octave(3)
            False

        ::

            >>> octave == 'foo'
            False

        Returns boolean.
        '''
        try:
            other = type(self)(other)
            return self.octave_number == other.octave_number
        except:
            return False

    def __float__(self):
        r'''Cast octave as floating-point number.

        ::

            >>> float(pitchtools.Octave(3))
            3.0

        Returns floating-point number.
        '''
        return float(self.octave_number)

    def __hash__(self):
        r'''Hashes octave.

        Returns integer.
        '''
        return hash(repr(self))

    def __int__(self):
        r'''Changes octave to integer.

        ::

            >>> int(pitchtools.Octave(3))
            3

        Returns integer.
        '''
        return self.octave_number

    def __str__(self):
        r'''String representation of octave.

        Defined equal to LilyPond octave / tick representation of octave.

        ::

            >>> str(pitchtools.Octave(4))
            "'"

        ::

            >>> str(pitchtools.Octave(1))
            ',,'

        ::

            >>> str(pitchtools.Octave(3))
            ''

        Returns string.
        '''
        if 3 < self.octave_number:
            return "'" * (self.octave_number - 3)
        elif self.octave_number < 3:
            return ',' * abs(3 - self.octave_number)
        return ''

    ### PRIVATE PROPERTIES ###

    @property
    def _storage_format_specification(self):
        from abjad.tools import systemtools
        return systemtools.StorageFormatSpecification(
            self,
            is_indented=False,
            keyword_argument_names=(),
            positional_argument_values=(
                self.octave_number,
                )
            )

    ### PUBLIC PROPERTIES ###

    @property
    def octave_number(self):
        r'''Octave number of octave.

        ::

            >>> pitchtools.Octave(5).octave_number
            5

        Returns integer.
        '''
        return self._octave_number

    @property
    def octave_tick_string(self):
        r"""LilyPond octave tick representation of octave.

        ::

            >>> for i in range(-1, 9):
            ...     print(i, pitchtools.Octave(i).octave_tick_string)
            -1 ,,,,
            0  ,,,
            1  ,,
            2  ,
            3
            4  '
            5  ''
            6  '''
            7  ''''
            8  '''''

        Returns string.
        """
        return str(self)

    @property
    def pitch_number(self):
        r'''Pitch number of first note in octave.

        ::

            >>> pitchtools.Octave(4).pitch_number
            0

        ::

            >>> pitchtools.Octave(5).pitch_number
            12

        ::

            >>> pitchtools.Octave(3).pitch_number
            -12

        Returns integer.
        '''
        return (self.octave_number - 4) * 12

    @property
    def pitch_range(self):
        r'''Pitch range of octave.

        ::

            >>> pitchtools.Octave(5).pitch_range
            PitchRange(range_string='[C5, C6)')

        Returns pitch range.
        '''
        from abjad.tools import pitchtools
        return pitchtools.PitchRange(
            '[C{}, C{})'.format(
                self.octave_number,
                self.octave_number + 1,
                ))

    ### PUBLIC METHODS ###

    @classmethod
    def from_pitch_name(cls, pitch_name):
        '''Makes octave from `pitch_name`.

        ::

            >>> pitchtools.Octave.from_pitch_name('cs')
            Octave(3)

        Returns integer.
        '''
        from abjad.tools import pitchtools
        if not isinstance(pitch_name, str):
            message = 'must be string: {!r}.'
            message = message.format(pitch_name)
            raise TypeError(message)
        match = re.match('^([a-z]+)(\,*|\'*)$', pitch_name)
        if match is None:
            message = 'incorrect pitch string format.'
            raise TypeError(message)
        name, tick_string = match.groups()
        return cls(tick_string)

    @classmethod
    def from_pitch_number(cls, pitch_number):
        r'''Makes octave from `pitch_number`.

        ::

            >>> pitchtools.Octave.from_pitch_number(13)
            Octave(5)

        Returns octave.
        '''
        octave_number = int(math.floor(pitch_number / 12)) + 4
        return cls(octave_number)

    @classmethod
    def is_octave_tick_string(cls, expr):
        '''Is true when `expr` is an octave tick string. Otherwise false.

        ::

            >>> pitchtools.Octave.is_octave_tick_string(',,,')
            True

        The regex ``^,+|'+|$`` underlies this predicate.

        Returns boolean.
        '''
        if not isinstance(expr, str):
            return False
        return bool(cls._octave_tick_regex.match(expr))


# -*- encoding: utf-8 -*-
import pytest
from abjad import *


def test_datastructuretools_StatalServerCursor___call___01():
    r'''Invalid levels raise an exception.
    '''

    sequence = [(0, 1), (2, 3), (4, 5), (6, 7)]
    server = datastructuretools.StatalServer(sequence)
    cursor = server()

    statement = 'cursor(level=-99)'
    assert pytest.raises(Exception, statement)


# -*- encoding: utf-8 -*-
import sys
if sys.version_info[0] == 2:
    import desordre
    import ferneyhough
    import mozart
    import part
else:
    from abjad.demos import desordre
    from abjad.demos import ferneyhough
    from abjad.demos import mozart
    from abjad.demos import part
del sys



# -*- encoding: utf-8 -*-
import fractions
import sys
from abjad import *


def test_mathtools_NonreducedFraction___init___01():

    if sys.version_info[0] == 2:

        result = systemtools.IOManager.count_function_calls(
            'fractions.Fraction(3, 6)', globals())
        assert result == 13

        result = systemtools.IOManager.count_function_calls(
            'mathtools.NonreducedFraction(3, 6)', globals())
        assert result == 30

    else:

        result = systemtools.IOManager.count_function_calls(
            'fractions.Fraction(3, 6)', globals())
        assert result == 12

        result = systemtools.IOManager.count_function_calls(
            'mathtools.NonreducedFraction(3, 6)', globals())
        assert result == 32



# -*- encoding: utf-8 -*-
import re


underscore_delimited_lowercase_regex_body = '(([a-z,0-9]+[_]+)*[a-z,0-9]+)?'
underscore_delimited_lowercase_regex = re.compile(
    '^%s$' % underscore_delimited_lowercase_regex_body)

def is_snake_case(expr):
    r'''Is true when `expr` is a string and is underscore delimited lowercase.

    ..  container:: example

        ::

            >>> stringtools.is_snake_case('foo_bar')
            True

    Otherwise false:

    ..  container:: example

        ::

            >>> stringtools.is_snake_case('foo bar')
            False

    Returns boolean.
    '''

    if not isinstance(expr, str):
        return False

    return bool(underscore_delimited_lowercase_regex.match(expr))


# -*- encoding: utf-8 -*-
from abjad import *


def test_pitchtools_PitchArray_from_score_01():

    score = Score([])
    score.append(Staff("c'8 d'8 e'8 f'8 g'8 a'8"))
    score.append(Staff("c'4 d'4 e'4"))
    score.append(Staff("c'8 d'8 e'8 f'8 g'8 a'8"))

    r'''
    \new Score <<
        \new Staff {
            c'8
            d'8
            e'8
            f'8
            g'8
            a'8
        }
        \new Staff {
            c'4
            d'4
            e'4
        }
        \new Staff {
            c'8
            d'8
            e'8
            f'8
            g'8
            a'8
        }
    >>
    '''

    pitch_array = pitchtools.PitchArray.from_score(score, populate=False)

    '''
    [] [] [] [] [] []
    [      ] [      ] [      ]
    [] [] [] [] [] []
    '''

    assert pitch_array[0].cell_widths == (1, 1, 1, 1, 1, 1)
    assert pitch_array[1].cell_widths == (2, 2, 2)
    assert pitch_array[2].cell_widths == (1, 1, 1, 1, 1, 1)


def test_pitchtools_PitchArray_from_score_02():

    score = Score([])
    score.append(Staff("c'8 d'8 e'8 f'8"))
    score.append(Staff("c'4 d'4"))
    score.append(Staff(scoretools.FixedDurationTuplet(Duration(2, 8), "c'8 d'8 e'8") * 2))

    r'''
    \new Score <<
        \new Staff {
            c'8
            d'8
            e'8
            f'8
        }
        \new Staff {
            c'4
            d'4
        }
        \new Staff {
            \times 2/3 {
                c'8
                d'8
                e'8
            }
            \times 2/3 {
                c'8
                d'8
                e'8
            }
        }
    >>
    '''

    pitch_array = pitchtools.PitchArray.from_score(score, populate=False)

    '''
    [      ] [      ] [      ] [      ]
    [                 ] [                 ]
    [] [      ] [] [] [      ] []
    '''

    assert pitch_array[0].cell_widths == (2, 2, 2, 2)
    assert pitch_array[1].cell_widths == (4, 4)
    assert pitch_array[2].cell_widths == (1, 2, 1, 1, 2, 1)


def test_pitchtools_PitchArray_from_score_03():

    score = Score([])
    score.append(Staff("c'8 d'8 e'8 f'8 g'8 a'8"))
    score.append(Staff("c'4 d'4 e'4"))
    score.append(Staff("c'8 d'8 e'8 f'8 g'8 a'8"))

    r'''
    \new Score <<
        \new Staff {
            c'8
            d'8
            e'8
            f'8
            g'8
            a'8
        }
        \new Staff {
            c'4
            d'4
            e'4
        }
        \new Staff {
            c'8
            d'8
            e'8
            f'8
            g'8
            a'8
        }
    >>
    '''

    pitch_array = pitchtools.PitchArray.from_score(score, populate=True)

    '''
    [c'] [d'] [e'] [f'] [g'] [a']
    [c'      ] [d'      ] [e'      ]
    [c'] [d'] [e'] [f'] [g'] [a']
    '''

    assert pitch_array[0].pitches ==\
        pitchtools.list_named_pitches_in_expr(score[0])
    assert pitch_array[1].pitches ==\
        pitchtools.list_named_pitches_in_expr(score[1])
    assert pitch_array[2].pitches ==\
        pitchtools.list_named_pitches_in_expr(score[2])


def test_pitchtools_PitchArray_from_score_04():

    score = Score([])
    score.append(Staff("c'8 d'8 e'8 f'8"))
    score.append(Staff("c'4 d'4"))
    score.append(
        Staff(
            scoretools.FixedDurationTuplet(Duration(2, 8), "c'8 d'8 e'8") * 2))

    r'''
    \new Score <<
        \new Staff {
            c'8
            d'8
            e'8
            f'8
        }
        \new Staff {
            c'4
            d'4
        }
        \new Staff {
            \times 2/3 {
                c'8
                d'8
                e'8
            }
            \times 2/3 {
                c'8
                d'8
                e'8
            }
        }
    >>
    '''

    pitch_array = pitchtools.PitchArray.from_score(score, populate=True)

    '''
    [c'      ] [d'      ] [e'      ] [f'      ]
    [c'             ] [d'             ]
    [c'] [d'      ] [e'] [c'] [d'      ] [e']
    '''

    assert pitch_array[0].pitches ==\
        pitchtools.list_named_pitches_in_expr(score[0])
    assert pitch_array[1].pitches ==\
        pitchtools.list_named_pitches_in_expr(score[1])
    assert pitch_array[2].pitches ==\
        pitchtools.list_named_pitches_in_expr(score[2])


# -*- encoding: utf-8 -*-
import abc
import os
from abjad.tools import abctools


class OutputFormat(abctools.AbjadObject):
    r'''Output format.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_code_block_closing',
        '_code_block_opening',
        '_code_indent',
        '_image_block',
        '_image_format',
        )

    ### INITIALIZER ###

    @abc.abstractmethod
    def __init__(
        self,
        code_block_opening,
        code_block_closing,
        code_indent,
        image_block,
        image_format,
        ):
        self._code_block_opening = code_block_opening
        self._code_block_closing = code_block_closing
        self._code_indent = code_indent
        self._image_block = image_block
        self._image_format = image_format

    ### SPECIAL METHODS ###

    def __call__(self, code_block, image_dict):
        r'''Calls output format with `code_block` and `image_dict`.
        '''

        reformatted = []
        for result in code_block.processed_results:
            if isinstance(result, tuple):
                reformatted.append(
                    self.code_block_opening +
                    '\n'.join([(' ' * self.code_indent) + x for x in result]) +
                    self.code_block_closing)
                for x in result:
                    if 'Error:' in x or 'Exception:' in x:
                        print('\nAbjadBookError:\n\n{}'.format(
                            '\n'.join(reformatted)))
                        break

            elif isinstance(result, dict):
                file_name = result['file_name']
                image_count = result['image_count']
                image_prefix = result['image_prefix']
                page_range = result['page_range']
                scale = result['scale']
                if scale is None:
                    scale = 1.0

                if page_range is None:
                    try:
                        image_file_names = [v for k, v in sorted(
                            image_dict[image_count].items())]
                        for image_file_name in image_file_names:
                            image_file_name = image_file_name.rpartition('.')[0]
                            image_block = self.image_block.format(
                                image_file_name=image_file_name,
                                scale=scale,
                                )
                            reformatted.append(image_block)
                    except KeyError:
                        print('\nAbjadBookError:\n\n{}'.format(
                            '\n'.join(reformatted)))

                else:
                    try:
                        for page_number, image_file_name in sorted(
                            image_dict[image_count].items()):
                            if page_number in page_range:
                                image_file_name = \
                                    image_file_name.rpartition('.')[0]
                                image_block = self.image_block.format(
                                    image_file_name=image_file_name,
                                    scale=scale,
                                    )
                                reformatted.append(image_block)
                            else:
                                os.remove(image_file_name)
                    except KeyError:
                        print('\nAbjadBookError:\n\n{}'.format(
                            '\n'.join(reformatted)))

        return tuple(reformatted)

    ### PUBLIC PROPERTIES ###

    @property
    def code_block_closing(self):
        r'''Code block closing.
        '''
        return self._code_block_closing

    @property
    def code_block_opening(self):
        r'''Code block opening.
        '''
        return self._code_block_opening

    @property
    def code_indent(self):
        r'''Code indent.
        '''
        return self._code_indent

    @property
    def image_block(self):
        r'''Image block.
        '''
        return self._image_block

    @property
    def image_format(self):
        r'''Image format.
        '''
        return self._image_format


# -*- encoding: utf-8 -*-
import abc
import os
from abjad.tools.abctools.AbjadObject import AbjadObject


class Documenter(AbjadObject):
    r'''Documenter is an abstract base class for documentation classes.
    '''

    ### INITIALIZER ###

    def __init__(self, subject=None, prefix='abjad.tools.'):
        assert isinstance(prefix, (str, type(None)))
        self._subject = subject
        self._prefix = prefix

    ### SPECIAL METHODS ###

    def __format__(self, format_specification=''):
        r'''Formats documenter.

        Set `format_specification` to `''` or `'storage'`.
        Interprets `''` equal to `'storage'`.

        Returns string.
        '''
        from abjad.tools import systemtools
        if format_specification in ('', 'storage'):
            return systemtools.StorageFormatManager.get_storage_format(self)
        return str(self)

    ### PRIVATE PROPERTIES ###

    @property
    def _storage_format_specification(self):
        from abjad.tools import systemtools
        keyword_argument_names = (
            'prefix',
            )
        if self.subject is type(None):
            positional_argument_values = ()
        else:
            positional_argument_values = (
                self.subject,
                )
        return systemtools.StorageFormatSpecification(
            self,
            positional_argument_values=positional_argument_values,
            keyword_argument_names=keyword_argument_names,
            )

    ### PRIVATE METHODS ###

    def _shrink_module_name(self, module):
        if self.prefix and module.startswith(self.prefix):
            module = module.partition(self.prefix)[-1]
        parts = module.split('.')
        unique = [parts[0]]
        for part in parts[1:]:
            if part != unique[-1]:
                unique.append(part)
        return '.'.join(unique)

    ### PUBLIC METHODS ###

    @staticmethod
    def write(file_path, restructured_text):
        r'''Writes `restructed_text` to `file_path`.

        Returns none.
        '''
        should_write = True
        if os.path.exists(file_path):
            with open(file_path, 'r') as f:
                if f.read() == restructured_text:
                    should_write = False
        if should_write:
            print('WRITING {}'.format(os.path.relpath(file_path)))
            with open(file_path, 'w') as f:
                f.write(restructured_text)

    ### PUBLIC PROPERTIES ###

    @property
    def module_name(self):
        r'''Module name of documenter.

        Returns string.
        '''
        return '{}.{}'.format(self.subject.__module__, self.subject.__name__)

    @property
    def prefix(self):
        r'''Prefix of documenter.
        '''
        return self._prefix

    @property
    def subject(self):
        r'''Object of documenter.
        '''
        return self._subject


# -*- encoding: utf-8 -*-
import itertools


def yield_all_compositions_of_integer(n):
    r'''Yields all compositions of positive integer `n`
    in descending lex order:

    ::

        >>> for integer_composition in mathtools.yield_all_compositions_of_integer(5):
        ...     integer_composition
        ...
        (5,)
        (4, 1)
        (3, 2)
        (3, 1, 1)
        (2, 3)
        (2, 2, 1)
        (2, 1, 2)
        (2, 1, 1, 1)
        (1, 4)
        (1, 3, 1)
        (1, 2, 2)
        (1, 2, 1, 1)
        (1, 1, 3)
        (1, 1, 2, 1)
        (1, 1, 1, 2)
        (1, 1, 1, 1, 1)

    Integer compositions are ordered integer partitions.

    Returns generator of positive integer tuples of length at least ``1``.
    '''
    from abjad.tools import mathtools

    # Finds small values of n easily.
    # Takes ca. 4 seconds for n = 17.

    compositions = []

    x = 0
    string_length = n
    while x < 2 ** (n - 1):
        binary_string = mathtools.integer_to_binary_string(x)
        binary_string = binary_string.zfill(string_length)
        l = [int(c) for c in list(binary_string)]
        partition = []
        g = itertools.groupby(l, lambda x: x)
        for value, group in g:
            partition.append(list(group))
        sublengths = [len(part) for part in partition]
        composition = tuple(sublengths)
        compositions.append(composition)
        x += 1

    for composition in reversed(sorted(compositions)):
        yield composition


# -*- encoding: utf-8 -*-
from abjad.tools import mathtools


def partition_sequence_by_weights(
    sequence,
    weights,
    cyclic=False,
    overhang=False,
    allow_part_weights=Exact,
    ):
    r'''Partitions `sequence` by `weights` exactly.

    ::

        >>> sequence = [3, 3, 3, 3, 4, 4, 4, 4, 5]

    ..  container:: example

        **Example 1.** Partition sequence once by weights exactly without
        overhang:

        ::

            >>> sequencetools.partition_sequence_by_weights(
            ...     sequence,
            ...     [3, 9],
            ...     cyclic=False,
            ...     overhang=False,
            ...     )
            [[3], [3, 3, 3]]

    ..  container:: example

        **Example 2.** Partition sequence once by weights exactly with
        overhang:

        ::

            >>> sequencetools.partition_sequence_by_weights(
            ...     sequence,
            ...     [3, 9],
            ...     cyclic=False,
            ...     overhang=True,
            ...     )
            [[3], [3, 3, 3], [4, 4, 4, 4, 5]]

    ..  container:: example

        **Example 3.** Partition sequence cyclically by weights exactly
        without overhang:

        ::

            >>> sequencetools.partition_sequence_by_weights(
            ...     sequence,
            ...     [12],
            ...     cyclic=True,
            ...     overhang=False,
            ...     )
            [[3, 3, 3, 3], [4, 4, 4]]

    ..  container:: example

        **Example 4.** Partition sequence cyclically by weights exactly with
        overhang:

        ::

            >>> sequencetools.partition_sequence_by_weights(
            ...     sequence,
            ...     [12],
            ...     cyclic=True,
            ...     overhang=True,
            ...     )
            [[3, 3, 3, 3], [4, 4, 4], [4, 5]]

    ::

        >>> sequence = [3, 3, 3, 3, 4, 4, 4, 4, 5, 5]

    ..  container:: example

        **Example 1.** Partition sequence once by weights at most without
        overhang:

        ::

            >>> sequencetools.partition_sequence_by_weights(
            ...     sequence,
            ...     [10, 4],
            ...     cyclic=False,
            ...     overhang=False,
            ...     allow_part_weights=Less,
            ...     )
            [[3, 3, 3], [3]]

    ..  container:: example

        **Example 2.** Partition sequence once by weights at most with
        overhang:

        ::

            >>> sequencetools.partition_sequence_by_weights(
            ...     sequence,
            ...     [10, 4],
            ...     cyclic=False,
            ...     overhang=True,
            ...     allow_part_weights=Less,
            ...     )
            [[3, 3, 3], [3], [4, 4, 4, 4, 5, 5]]

    ..  container:: example

        **Example 3.** Partition sequence cyclically by weights at most
        without overhang:

        ::

            >>> sequencetools.partition_sequence_by_weights(
            ...     sequence,
            ...     [10, 5],
            ...     cyclic=True,
            ...     overhang=False,
            ...     allow_part_weights=Less,
            ...     )
            [[3, 3, 3], [3], [4, 4], [4], [4, 5], [5]]

    ..  container:: example

        **Example 4.** Partition sequence cyclically by weights at most
        with overhang:

        ::

            >>> sequencetools.partition_sequence_by_weights(
            ...     sequence,
            ...     [10, 5],
            ...     cyclic=True,
            ...     overhang=True,
            ...     allow_part_weights=Less,
            ...     )
            [[3, 3, 3], [3], [4, 4], [4], [4, 5], [5]]

    ::

        >>> sequence = [3, 3, 3, 3, 4, 4, 4, 4, 5, 5]

    ..  container:: example

        **Example 1.** Partition sequence once by weights at least without
        overhang:

        ::

            >>> sequencetools.partition_sequence_by_weights(
            ...     sequence,
            ...     [10, 4],
            ...     cyclic=False,
            ...     overhang=False,
            ...     allow_part_weights=More,
            ...     )
            [[3, 3, 3, 3], [4]]

    ..  container:: example

        **Example 2.** Partition sequence once by weights at least with
        overhang:

        ::

            >>> sequencetools.partition_sequence_by_weights(
            ...     sequence,
            ...     [10, 4],
            ...     cyclic=False,
            ...     overhang=True,
            ...     allow_part_weights=More,
            ...     )
            [[3, 3, 3, 3], [4], [4, 4, 4, 5, 5]]

    ..  container:: example

        **Example 3.** Partition sequence cyclically by weights at least
        without overhang:

        ::

            >>> sequencetools.partition_sequence_by_weights(
            ...     sequence,
            ...     [10, 4],
            ...     cyclic=True,
            ...     overhang=False,
            ...     allow_part_weights=More,
            ...     )
            [[3, 3, 3, 3], [4], [4, 4, 4], [5]]

    ..  container:: example

        **Example 4.** Partition sequence cyclically by weights at least with
        overhang:

        ::

            >>> sequencetools.partition_sequence_by_weights(
            ...     sequence,
            ...     [10, 4],
            ...     cyclic=True,
            ...     overhang=True,
            ...     allow_part_weights=More,
            ...     )
            [[3, 3, 3, 3], [4], [4, 4, 4], [5], [5]]

    Returns list sequence objects.
    '''
    from abjad.tools import sequencetools

    if allow_part_weights == Exact:
        candidate = sequencetools.split_sequence(
            sequence,
            weights,
            cyclic=cyclic,
            overhang=overhang,
            )
        flattened_candidate = sequencetools.flatten_sequence(candidate)
        if flattened_candidate == sequence[:len(flattened_candidate)]:
            return candidate
        else:
            message = 'can not partition exactly.'
            raise PartitionError(message)
    elif allow_part_weights == More:
        if not cyclic:
            return _partition_sequence_once_by_weights_at_least(
                sequence, weights, overhang=overhang)
        else:
            return _partition_sequence_cyclically_by_weights_at_least(
                sequence, weights, overhang=overhang)
    elif allow_part_weights == Less:
        if not cyclic:
            return _partition_sequence_once_by_weights_at_most(
                sequence,
                weights,
                overhang=overhang,
                )
        else:
            return _partition_sequence_cyclically_by_weights_at_most(
                sequence,
                weights,
                overhang=overhang,
                )
    else:
        message = 'not an ordinal value constant: {!r}.'
        message = message.format(allow_part_weights)
        raise ValueError(message)


def _partition_sequence_once_by_weights_at_least(
    sequence,
    weights,
    overhang=False,
    ):
    result = []
    current_part = []
    l_copy = sequence[:]
    for num_weight, target_weight in enumerate(weights):
        while True:
            try:
                x = l_copy.pop(0)
            except IndexError:
                if num_weight + 1 == len(weights):
                    if current_part:
                        result.append(current_part)
                        break
                message = 'too few elements in sequence.'
                raise PartitionError(message)
            current_part.append(x)
            if target_weight <= mathtools.weight(current_part):
                result.append(current_part)
                current_part = []
                break
    if l_copy:
        if overhang:
            result.append(l_copy)
    return result


def _partition_sequence_cyclically_by_weights_at_least(
    sequence,
    weights,
    overhang=False,
    ):
    l_copy = sequence[:]
    result = []
    current_part = []
    target_weight_index = 0
    len_weights = len(weights)
    while l_copy:
        target_weight = weights[target_weight_index % len_weights]
        x = l_copy.pop(0)
        current_part.append(x)
        if target_weight <= mathtools.weight(current_part):
            result.append(current_part)
            current_part = []
            target_weight_index += 1
    assert not l_copy
    if current_part:
        if overhang:
            result.append(current_part)
    return result


def _partition_sequence_once_by_weights_at_most(
    sequence,
    weights,
    overhang=False,
    ):
    l_copy = sequence[:]
    result = []
    current_part = []
    for target_weight in weights:
        while True:
            try:
                x = l_copy.pop(0)
            except IndexError:
                message = 'too few elements in sequence.'
                raise PartitionError(message)
            current_weight = mathtools.weight(current_part)
            candidate_weight = current_weight + mathtools.weight([x])
            if candidate_weight < target_weight:
                current_part.append(x)
            elif candidate_weight == target_weight:
                current_part.append(x)
                result.append(current_part)
                current_part = []
                break
            elif target_weight < candidate_weight:
                if current_part:
                    result.append(current_part)
                    current_part = []
                    l_copy.insert(0, x)
                    break
                else:
                    message = 'elements in sequence too big.'
                    raise PartitionError(message)
            else:
                message = 'candidate and target weights must compare.'
                raise ValueError(message)
    if overhang:
        left_over = current_part + l_copy
        if left_over:
            result.append(left_over)
    return result


def _partition_sequence_cyclically_by_weights_at_most(
    sequence,
    weights,
    overhang=False,
    ):
    result = []
    current_part = []
    current_target_weight_index = 0
    current_target_weight = weights[current_target_weight_index]
    l_copy = sequence[:]
    while l_copy:
        current_target_weight = \
            weights[current_target_weight_index % len(weights)]
        x = l_copy.pop(0)
        current_part_weight = mathtools.weight(current_part)
        candidate_part_weight = current_part_weight + mathtools.weight([x])
        if candidate_part_weight < current_target_weight:
            current_part.append(x)
        elif candidate_part_weight == current_target_weight:
            current_part.append(x)
            result.append(current_part)
            current_part = []
            current_target_weight_index += 1
        elif current_target_weight < candidate_part_weight:
            if current_part:
                l_copy.insert(0, x)
                result.append(current_part)
                current_part = []
                current_target_weight_index += 1
            else:
                message = 'elements in sequence too big.'
                raise PartitionError(message)
        else:
            message = 'candidate and target rates must compare.'
            raise ValueError(message)
    if current_part:
        if overhang:
            result.append(current_part)
    return result


# -*- encoding: utf-8 -*-
from abjad.tools import durationtools
from abjad.tools import mathtools
from abjad.tools import selectiontools
from abjad.tools import sequencetools
from abjad.tools.topleveltools import inspect_
from abjad.tools.topleveltools import mutate
Selection = selectiontools.Selection


def _rewrite_meter(
    components,
    meter,
    boundary_depth=None,
    initial_offset=None,
    maximum_dot_count=None,
    ):
    from abjad.tools import metertools

    assert isinstance(components, selectiontools.Selection), \
        repr(components)

    def get_offsets_at_depth(depth):
        if depth < len(offset_inventory):
            return offset_inventory[depth]
        while len(offset_inventory) <= depth:
            new_offsets = []
            old_offsets = offset_inventory[-1]
            for first, second in \
                sequencetools.iterate_sequence_nwise(old_offsets):
                new_offsets.append(first)
                difference = second - first
                half = (first + second) / 2
                if durationtools.Duration(1, 8) < difference:
                    new_offsets.append(half)
                else:
                    one_quarter = (first + half) / 2
                    three_quarters = (half + second) / 2
                    new_offsets.append(one_quarter)
                    new_offsets.append(half)
                    new_offsets.append(three_quarters)
            new_offsets.append(old_offsets[-1])
            offset_inventory.append(tuple(new_offsets))
        return offset_inventory[depth]

    def recurse(
        boundary_depth=None,
        boundary_offsets=None,
        depth=0,
        logical_tie=None,
        ):
        offsets = get_offsets_at_depth(depth)
        #print 'DEPTH:', depth

        logical_tie_duration = logical_tie._preprolated_duration
        logical_tie_timespan = logical_tie.get_timespan()
        logical_tie_start_offset = logical_tie_timespan.start_offset
        logical_tie_stop_offset = logical_tie_timespan.stop_offset
        logical_tie_starts_in_offsets = logical_tie_start_offset in offsets
        logical_tie_stops_in_offsets = logical_tie_stop_offset in offsets

        if not metertools.MeterManager.is_acceptable_logical_tie(
            logical_tie_duration=logical_tie_duration,
            logical_tie_starts_in_offsets=logical_tie_starts_in_offsets,
            logical_tie_stops_in_offsets=logical_tie_stops_in_offsets,
            maximum_dot_count=maximum_dot_count,
            ):

            #print 'UNACCEPTABLE:', logical_tie, logical_tie_start_offset, logical_tie_stop_offset
            #print '\t', ' '.join([str(x) for x in offsets])
            split_offset = None
            offsets = get_offsets_at_depth(depth)

            # If the logical tie's start aligns, take the latest possible offset.
            if logical_tie_starts_in_offsets:
                offsets = reversed(offsets)

            for offset in offsets:
                if logical_tie_start_offset < offset < logical_tie_stop_offset:
                    split_offset = offset
                    break

            #print '\tABS:', split_offset
            if split_offset is not None:
                split_offset -= logical_tie_start_offset
                #print '\tREL:', split_offset
                #print ''
                shards = mutate(logical_tie[:]).split([split_offset])
                logical_ties = \
                    [selectiontools.LogicalTie(shard) for shard in shards]
                for logical_tie in logical_ties:
                    recurse(
                        boundary_depth=boundary_depth,
                        boundary_offsets=boundary_offsets,
                        depth=depth,
                        logical_tie=logical_tie,
                        )
            else:
                #print ''
                recurse(
                    boundary_depth=boundary_depth,
                    boundary_offsets=boundary_offsets,
                    depth=depth + 1,
                    logical_tie=logical_tie,
                    )

        elif metertools.MeterManager.is_boundary_crossing_logical_tie(
            boundary_depth=boundary_depth,
            boundary_offsets=boundary_offsets,
            logical_tie_start_offset=logical_tie_start_offset,
            logical_tie_stop_offset=logical_tie_stop_offset,
            ):

            #print 'BOUNDARY CROSSING', logical_tie, logical_tie_start_offset, logical_tie_stop_offset
            offsets = boundary_offsets
            if logical_tie_start_offset in boundary_offsets:
                offsets = reversed(boundary_offsets)
            split_offset = None
            for offset in offsets:
                if logical_tie_start_offset < offset < logical_tie_stop_offset:
                    split_offset = offset
                    break
            assert split_offset is not None
            #print '\tABS:', split_offset
            split_offset -= logical_tie_start_offset
            #print '\tREL:', split_offset
            #print ''
            shards = mutate(logical_tie[:]).split([split_offset])
            logical_ties = \
                [selectiontools.LogicalTie(shard) for shard in shards]
            for logical_tie in logical_ties:
                recurse(
                    boundary_depth=boundary_depth,
                    boundary_offsets=boundary_offsets,
                    depth=depth,
                    logical_tie=logical_tie,
                    )

        else:
            #print 'ACCEPTABLE:', logical_tie, logical_tie_start_offset, logical_tie_stop_offset
            #print '\t', ' '.join([str(x) for x in offsets])
            #print ''
            logical_tie[:]._fuse()

    # Validate arguments.
    assert Selection._all_are_contiguous_components_in_same_logical_voice(
        components)
    if not isinstance(meter,
        metertools.Meter):
        meter = \
            metertools.Meter(meter)

    #assert sum([x._preprolated_duration for x in components]) == \
    #    meter.preprolated_duration
    if boundary_depth is not None:
        boundary_depth = int(boundary_depth)
    if maximum_dot_count is not None:
        maximum_dot_count = int(maximum_dot_count)
        assert 0 <= maximum_dot_count

    if initial_offset is None:
        initial_offset = durationtools.Offset(0)
    initial_offset = durationtools.Offset(initial_offset)

    first_start_offset = inspect_(components[0]).get_timespan().start_offset
    last_start_offset = inspect_(components[-1]).get_timespan().start_offset
    difference = last_start_offset - first_start_offset + initial_offset
    assert difference < meter.implied_time_signature.duration

    # Build offset inventory, adjusted for initial offset and prolation.
    first_offset = components[0]._get_timespan().start_offset
    first_offset -= initial_offset
    prolation = components[0]._get_parentage(include_self=False).prolation
    offset_inventory = []
    for offsets in meter.depthwise_offset_inventory:
        offsets = [(x * prolation) + first_offset for x in offsets]
        offset_inventory.append(tuple(offsets))

    # Build boundary offset inventory, if applicable.
    if boundary_depth is not None:
        boundary_offsets = offset_inventory[boundary_depth]
    else:
        boundary_offsets = None

    # Cache results of iterator, as we'll be mutating the underlying collection
    iterator = metertools.MeterManager.iterate_rewrite_inputs(components)
    items = tuple(iterator)
    for item in items:
        if isinstance(item, selectiontools.LogicalTie):
            #print 'RECURSING:', item
            recurse(
                boundary_depth=boundary_depth,
                boundary_offsets=boundary_offsets,
                depth=0,
                logical_tie=item,
                )
        else:
            #print 'DESCENDING:', item
            preprolated_duration = sum([x._preprolated_duration for x in item])
            if preprolated_duration.numerator == 1:
                preprolated_duration = mathtools.NonreducedFraction(
                    preprolated_duration)
                preprolated_duration = preprolated_duration.with_denominator(
                    preprolated_duration.denominator * 4)
            sub_metrical_hierarchy = metertools.Meter(preprolated_duration)
            sub_boundary_depth = 1
            if boundary_depth is None:
                sub_boundary_depth = None
            _rewrite_meter(
                item[:],
                sub_metrical_hierarchy,
                boundary_depth=sub_boundary_depth,
                maximum_dot_count=maximum_dot_count,
                )


# -*- encoding: utf-8 -*-


def retain_elements(
    sequence,
    indices=None,
    period=None,
    ):
    '''Retains `sequence` elements.

    ..  container:: example

        Retains all elements:

        ::

            >>> sequencetools.retain_elements(range(15))
            [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14]

    ..  container:: example

        Retains elements at indices 2 and 3:

        ::

            >>> sequencetools.retain_elements(
            ...     range(15),
            ...     indices=[2, 3],
            ...     )
            [2, 3]

    ..  container:: example

        Retains elements at indices -2 and -3:

        ::

            >>> sequencetools.retain_elements(
            ...     range(15),
            ...     indices=[-2, -3],
            ...     )
            [12, 13]

    ..  container:: example

        Retains elements at indices congruent to 2 or 3 (mod 4):

        ::

            >>> sequencetools.retain_elements(
            ...     range(15),
            ...     indices=[2, 3],
            ...     period=4,
            ...     )
            [2, 3, 6, 7, 10, 11, 14]

    ..  container:: example

        Retains elements at indices congruent to -2 or -3 (mod 4):

        ::

            >>> sequencetools.retain_elements(
            ...     range(15),
            ...     indices=[-2, -3],
            ...     period=4,
            ...     )
            [0, 1, 4, 5, 8, 9, 12, 13]

    ..  container:: example

        Retains no elements:

        ::

            >>> sequencetools.retain_elements(
            ...     range(15),
            ...     indices=[],
            ...     )
            []

    ..  container:: example

        Retains no elements:

        ::

            >>> sequencetools.retain_elements(
            ...     range(15),
            ...     indices=[97, 98, 99],
            ...     )
            []

    ..  container:: example

        Retains no elements:

        ::

            >>> sequencetools.retain_elements(
            ...     range(15),
            ...     indices=[-97, -98, -99],
            ...     )
            []

    Returns elements in the order they appear in `sequence`.

    Returns list.
    '''

    length = len(sequence)
    period = period or length

    if indices is None:
        indices = range(length)

    new_indices = []
    for i in indices:
        if length < abs(i):
            continue
        if i < 0:
            i = length + i
        i = i % period
        new_indices.append(i)

    indices = new_indices
    indices.sort()

    result = []

    for i, element in enumerate(sequence):
        if i % period in indices:
            result.append(element)

    return result


# -*- encoding: utf-8 -*-
from abjad import *


def test_markuptools_combine_markup_commands_01():

    markup_a = markuptools.MarkupCommand(
        'draw-circle', 4, 0.4, False)
    markup_b = markuptools.MarkupCommand(
        'filled-box',
        schemetools.SchemePair(-4, 4),
        schemetools.SchemePair(-0.5, 0.5),
        1,
        )
    markup_c = "some text"
    new_markup = markuptools.combine_markup_commands(
        markup_a, markup_b, markup_c)

    assert systemtools.TestManager.compare(
        format(new_markup, 'lilypond'),
        r'''
        \combine
            \combine
                \draw-circle
                    #4
                    #0.4
                    ##f
                \filled-box
                    #'(-4 . 4)
                    #'(-0.5 . 0.5)
                    #1
            "some text"
        ''',
        )


def test_markuptools_combine_markup_commands_02():

    markup_a = 'only a little text'
    assert markuptools.combine_markup_commands(markup_a) == markup_a


# -*- encoding: utf-8 -*-


def is_integer_n_tuple(expr, n):
    r'''Is true when `expr` is an integer tuple of length `n`.

    ::

        >>> mathtools.is_integer_n_tuple((19, 20, 21), 3)
        True

    Otherwise false:

    ::

        >>> mathtools.is_integer_n_tuple((19, 20, 'text'), 3)
        False

    Returns boolean.
    '''

    return isinstance(expr, tuple) and len(expr) == n and \
        all(isinstance(x, int) for x in expr)


# -*- encoding: utf-8 -*-
from abjad.tools import pitchtools
from abjad.tools.spannertools.Spanner import Spanner
from abjad.tools.topleveltools import override


class TrillSpanner(Spanner):
    r'''A trill spanner.

    ..  container:: example

        ::

            >>> staff = Staff("c'8 d'8 e'8 f'8")
            >>> trill = spannertools.TrillSpanner()
            >>> attach(trill, staff[:])
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> print(format(staff))
            \new Staff {
                c'8 \startTrillSpan
                d'8
                e'8
                f'8 \stopTrillSpan
            }

    Formats LilyPond ``\startTrillSpan`` on first leaf in spanner.

    Formats LilyPond ``\stopTrillSpan`` on last leaf in spanner.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_pitch',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        overrides=None,
        pitch=None,
        ):
        Spanner.__init__(
            self,
            overrides=overrides,
            )
        if pitch is not None:
            pitch = pitchtools.NamedPitch(pitch)
        self._pitch = pitch

    ### PRIVATE METHODS ###

    def _copy_keyword_args(self, new):
        new._pitch = self.pitch

    def _get_lilypond_format_bundle(self, leaf):
        from abjad.tools import systemtools
        lilypond_format_bundle = systemtools.LilyPondFormatBundle()
        if self._is_my_first_leaf(leaf):
            contributions = override(self)._list_format_contributions(
                'override',
                is_once=False,
                )
            lilypond_format_bundle.grob_overrides.extend(contributions)
            string = r'\startTrillSpan'
            lilypond_format_bundle.right.spanner_starts.append(string)
            if self.pitch is not None:
                string = r'\pitchedTrill'
                lilypond_format_bundle.opening.spanners.append(string)
                string = str(self.pitch)
                lilypond_format_bundle.right.spanner_starts.append(string)
        if self._is_my_last_leaf(leaf):
            contributions = override(self)._list_format_contributions(
                'revert',
                )
            lilypond_format_bundle.grob_reverts.extend(contributions)
            string = r'\stopTrillSpan'
            lilypond_format_bundle.right.spanner_stops.append(string)
        return lilypond_format_bundle

    ### PUBLIC PROPERTIES ###

    @property
    def pitch(self):
        r'''Gets optional pitch of trill spanner.

        ..  container:: example

            ::

                >>> staff = Staff("c'8 d'8 e'8 f'8")
                >>> pitch = NamedPitch('C#4')
                >>> trill = spannertools.TrillSpanner(pitch=pitch)
                >>> attach(trill, staff[:2])
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> print(format(staff))
                \new Staff {
                    \pitchedTrill c'8 \startTrillSpan cs'
                    d'8 \stopTrillSpan
                    e'8
                    f'8
                }

            ::

                >>> trill.pitch
                NamedPitch("cs'")

        Formats LilyPond ``\pitchedTrill`` command on first leaf in spanner.

        Returns named pitch or none.
        '''
        return self._pitch

    @property
    def written_pitch(self):
        r'''Gets written pitch of trill spanner.

        ..  container:: example

            ::

                >>> staff = Staff("c'8 d'8 e'8 f'8")
                >>> pitch = NamedPitch('C#4')
                >>> trill = spannertools.TrillSpanner(pitch=pitch)
                >>> attach(trill, staff[:2])
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> print(format(staff))
                \new Staff {
                    \pitchedTrill c'8 \startTrillSpan cs'
                    d'8 \stopTrillSpan
                    e'8
                    f'8
                }

            ::

                >>> trill.written_pitch
                NamedPitch("cs'")

        Defined equal to `pitch`.

        Returns named pitch or none.
        '''
        return self.pitch



# -*- encoding: utf-8 -*-


def repeat_elements(sequence, indices=None, period=None, total=1):
    '''Repeats `sequence` elements.

    ..  container:: example

        Repeats elements at indices 1 and 2 a total of three times each:

        ::

            >>> sequencetools.repeat_elements(
            ...     list(range(10)),
            ...     indices=[1, 2],
            ...     total=3,
            ...     )
            [0, [1, 1, 1], [2, 2, 2], 3, 4, 5, 6, 7, 8, 9]

    ..  container:: example

        Repeats elements at indices -1 and -2 a total of three times each:

        ::

            >>> sequencetools.repeat_elements(
            ...     list(range(10)),
            ...     indices=[-1, -2],
            ...     total=3,
            ...     )
            [0, 1, 2, 3, 4, 5, 6, 7, [8, 8, 8], [9, 9, 9]]

    ..  container:: example

        Repeats elements at indices congruent to 1 and 2 (mod 5) a total of
        three times each:

        ::

            >>> sequencetools.repeat_elements(
            ...     list(range(10)),
            ...     indices=[1, 2],
            ...     total=3,
            ...     period=5,
            ...     )
            [0, [1, 1, 1], [2, 2, 2], 3, 4, 5, [6, 6, 6], [7, 7, 7], 8, 9]

    ..  container:: example

        Repeats elements at indices congruent to -1 and -2 (mod 5) a total of
        three times each:

        ::

            >>> sequencetools.repeat_elements(
            ...     list(range(10)),
            ...     indices=[-1, -2],
            ...     total=3,
            ...     period=5,
            ...     )
            [0, 1, 2, [3, 3, 3], [4, 4, 4], 5, 6, 7, [8, 8, 8], [9, 9, 9]]

    ..  container:: example

        Repeats all elements a total of two times each:

        ::

            >>> sequencetools.repeat_elements(
            ...     list(range(10)),
            ...     total=2,
            ...     )
            [[0, 0], [1, 1], [2, 2], [3, 3], [4, 4], [5, 5], [6, 6], [7, 7], [8, 8], [9, 9]]

    ..  container:: example

        Repeats all elements a total of one time each:

        ::

            >>> sequencetools.repeat_elements(
            ...     list(range(10)),
            ...     total=1,
            ...     )
            [[0], [1], [2], [3], [4], [5], [6], [7], [8], [9]]

    ..  container:: example

        Repeats all elements a total of zero times each:

        ::

            >>> sequencetools.repeat_elements(
            ...     list(range(10)),
            ...     total=0,
            ...     )
            [[], [], [], [], [], [], [], [], [], []]

    ..  container:: example

        Repeats no elements:

        ::

            >>> sequencetools.repeat_elements(
            ...     list(range(10)),
            ...     indices=[],
            ...     )
            [0, 1, 2, 3, 4, 5, 6, 7, 8, 9]

    Returns new object of `sequence` type.
    '''

    length = len(sequence)
    period = period or length

    if indices is None:
        indices = range(length)

    new_indices = []
    for i in indices:
        if length < abs(i):
            continue
        if i < 0:
            i = length + i
        i = i % period
        new_indices.append(i)

    indices = new_indices
    indices.sort()

    result = []

    for i, element in enumerate(sequence):
        if i % period in indices:
            result.append(total * [element])
        else:
            result.append(element)

    result = type(sequence)(result)
    return result



# -*- encoding: utf-8 -*-
from abjad.tools.abctools.AbjadObject import AbjadObject


class LaissezVibrer(AbjadObject):
    r'''A *laissez vibrer* indication.

    ::

        >>> chord = Chord("<c' e' g' c''>4")
        >>> laissez_vibrer = indicatortools.LaissezVibrer()
        >>> attach(laissez_vibrer, chord)
        >>> show(chord) # doctest: +SKIP

    ..  doctest::

        >>> print(format(chord))
        <c' e' g' c''>4 \laissezVibrer

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        )

    _format_slot = 'right'

    ### INITIALIZER ###

    def __init__(self):
        pass

    ### SPECIAL METHODS ###

    def __eq__(self, expr):
        r'''Is true when `expr` is a *laissez vibrer* indication. Otherwise
        false.

        Returns boolean.
        '''
        return isinstance(expr, type(self))

    def __hash__(self):
        r'''Hashes laissez vibrer.

        Required to be explicitely re-defined on Python 3 if __eq__ changes.

        Returns integer.
        '''
        return super(LaissezVibrer, self).__hash__()

    def __str__(self):
        r'''String representation of laissez vibrer.

        Returns string.
        '''
        return r'\laissezVibrer'

    ### PRIVATE PROPERTIES ###

    @property
    def _lilypond_format(self):
        return str(self)

    @property
    def _lilypond_format_bundle(self):
        from abjad.tools import systemtools
        lilypond_format_bundle = systemtools.LilyPondFormatBundle()
        lilypond_format_bundle.right.articulations.append(str(self))
        return lilypond_format_bundle



# -*- encoding: utf-8 -*-
from abjad import *


def test_datastructuretools_CyclicMatrix_columns_01():

    cyclic_matrix = datastructuretools.CyclicMatrix([
        [0, 1, 2, 3],
        [10, 11, 12, 13],
        [20, 21, 22, 23],
        ])

    assert cyclic_matrix.columns == \
        ((0, 10, 20), (1, 11, 21), (2, 12, 22), (3, 13, 23))
    assert cyclic_matrix.columns[2] == (2, 12, 22)
    assert cyclic_matrix.columns[2][0] == 2


def test_datastructuretools_CyclicMatrix_columns_02():

    cyclic_matrix = datastructuretools.CyclicMatrix([
        [0, 1, 2, 3],
        [10, 11, 12, 13],
        [20, 21, 22, 23],
        ])

    assert cyclic_matrix.columns == \
        ((0, 10, 20), (1, 11, 21), (2, 12, 22), (3, 13, 23))
    assert cyclic_matrix.columns[99] == (3, 13, 23)
    assert cyclic_matrix.columns[99][99] == 3


# -*- encoding: utf-8 -*-
from abjad.tools.documentationtools.GraphvizObject import GraphvizObject


class GraphvizEdge(GraphvizObject):
    r'''A Graphviz edge.
    '''

    ### INITIALIZER ###

    def __init__(self, attributes=None, is_directed=True):
        GraphvizObject.__init__(self, attributes=attributes)
        self._head = None
        self._tail = None
        self._is_directed = bool(is_directed)
        self._head_port_position = None
        self._tail_port_position = None

    ### SPECIAL METHODS ###

    def __call__(self, *args):
        r'''Calls Graphviz edge.

        Returns Graphviz edge.
        '''
        from abjad.tools import documentationtools
        if args:
            assert len(args) == 2
            prototype = (
                documentationtools.GraphvizSubgraph,
                documentationtools.GraphvizNode,
                documentationtools.GraphvizField,
                )
            assert all(isinstance(x, prototype) for x in args)
            tail, head = args
            self._disconnect()
            self._connect(tail, head)
        else:
            self._disconnect()
        return self

    ### PRIVATE METHODS ###

    def _connect(self, tail, head):
        tail._edges.add(self)
        head._edges.add(self)
        self._tail = tail
        self._head = head

    def _disconnect(self):
        if self.tail is not None:
            self.tail._edges.remove(self)
        if self.head is not None:
            self.head._edges.remove(self)
        self._tail = None
        self._head = None

    ### PRIVATE PROPERTIES ###

    @property
    def _graphviz_format_contributions(self):
        connection = '->'
        if not self.is_directed:
            connection = '--'
        tail_name = self.tail.canonical_name
        if self._tail_port_position is not None:
            tail_name = '{}:{}'.format(tail_name, self._tail_port_position)
        head_name = self.head.canonical_name
        if self._head_port_position is not None:
            head_name = '{}:{}'.format(head_name, self._head_port_position)
        edge_def = '{} {} {}'.format(
            self._format_value(tail_name),
            connection,
            self._format_value(head_name),
            )
        if len(self.attributes):
            result = self._format_attribute_list(self.attributes)
            result[0] = '{} {}'.format(edge_def, result[0])
            return result
        return [edge_def + ';']

    ### PUBLIC PROPERTIES ###

    @property
    def head(self):
        r'''Head of Graphviz edge.
        '''
        return self._head

    @property
    def head_port_position(self):
        r'''Head port position.
        '''
        return self._head_port_position

    @head_port_position.setter
    def head_port_position(self, expr):
        self._head_port_position = expr

    @property
    def is_directed(self):
        r'''Is true when Graphviz edge is directed. Otherwise false.

        Returns boolean.
        '''
        return self._is_directed

    @is_directed.setter
    def is_directed(self, arg):
        self._is_directed = bool(arg)

    @property
    def tail(self):
        r'''Tail of Graphviz edge.
        '''
        return self._tail

    @property
    def tail_port_position(self):
        r'''Tail port position.
        '''
        return self._tail_port_position

    @tail_port_position.setter
    def tail_port_position(self, expr):
        self._tail_port_position = expr




# -*- encoding: utf-8 -*-
from abjad.tools import durationtools
from abjad.tools.abctools import AbjadValueObject


class DurationSpellingSpecifier(AbjadValueObject):
    r'''Duration spelling speficifier.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_decrease_durations_monotonically',
        '_forbidden_written_duration',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        decrease_durations_monotonically=True,
        forbidden_written_duration=None,
        ):
        assert isinstance(decrease_durations_monotonically, bool)
        if forbidden_written_duration is not None:
            forbidden_written_duration = durationtools.Duration(
                forbidden_written_duration)
        self._decrease_durations_monotonically = decrease_durations_monotonically
        self._forbidden_written_duration = forbidden_written_duration

    ### SPECIAL METHODS ###

    def __eq__(self, arg):
        r'''Is true when `arg` is a duration spelling specifier with
        values of `decrease_durations_monotonically` and `forbidden_written_duration`
        equal to those of this duration spelling specifier. Otherwise false.

        ..  container:: example

            ::

                >>> specifier_1 = rhythmmakertools.DurationSpellingSpecifier(
                ...     decrease_durations_monotonically=True,
                ...     )
                >>> specifier_2 = rhythmmakertools.DurationSpellingSpecifier(
                ...     decrease_durations_monotonically=False,
                ...     )

            ::

                >>> specifier_1 == specifier_1
                True
                >>> specifier_1 == specifier_2
                False
                >>> specifier_2 == specifier_1
                False
                >>> specifier_2 == specifier_2
                True

        Returns boolean.
        '''
        if isinstance(arg, type(self)):
            if self.decrease_durations_monotonically == arg.decrease_durations_monotonically and \
                self.forbidden_written_duration == \
                arg.forbidden_written_duration:
                return True
        return False

    def __format__(self, format_specification=''):
        r'''Formats duration spelling specifier.

        ..  container:: example

            ::

                >>> specifier = rhythmmakertools.DurationSpellingSpecifier()
                >>> print(format(specifier))
                rhythmmakertools.DurationSpellingSpecifier(
                    decrease_durations_monotonically=True,
                    )

        Returns string.
        '''
        return AbjadValueObject.__format__(
            self,
            format_specification=format_specification,
            )

    def __hash__(self):
        r'''Hashes duration spelling specifier.

        Required to be explicitely re-defined on Python 3 if __eq__ changes.

        Returns integer.
        '''
        return super(DurationSpellingSpecifier, self).__hash__()

    def __repr__(self):
        r'''Gets interpreter representation of duration spelling specifier.

        ..  container:: example

            ::

                >>> rhythmmakertools.DurationSpellingSpecifier()
                DurationSpellingSpecifier(decrease_durations_monotonically=True)

        Returns string.
        '''
        return AbjadValueObject.__repr__(self)

    ### PRIVATE PROPERTIES ###

    @property
    def _attribute_manifest(self):
        from abjad.tools import systemtools
        from scoremanager import idetools
        return systemtools.AttributeManifest(
            systemtools.AttributeDetail(
                name='decrease_durations_monotonically',
                command='ddm',
                editor=idetools.getters.get_boolean,
                ),
            systemtools.AttributeDetail(
                name='forbidden_written_duration',
                command='fwd',
                editor=idetools.getters.get_duration,
                ),
            )

    ### PUBLIC PROPERTIES ###

    @property
    def decrease_durations_monotonically(self):
        r'''Is true when all durations should be spelled as a tied series of
        monotonically decreasing values. Otherwise false.

        ..  container:: example

            ::

                >>> specifier = rhythmmakertools.DurationSpellingSpecifier()
                >>> specifier.decrease_durations_monotonically
                True

        Defaults to true.

        Returns boolean.
        '''
        return self._decrease_durations_monotonically

    @property
    def forbidden_written_duration(self):
        r'''Gets forbidden written duration.

        ..  container:: example

            ::

                >>> specifier = rhythmmakertools.DurationSpellingSpecifier()
                >>> specifier.forbidden_written_duration is None
                True

        Defaults to none.

        Returns duration or none.
        '''
        return self._forbidden_written_duration

    ### PUBLIC METHODS ###

    def reverse(self):
        r'''Reverses duration spelling specifier.

        ..  container:: example

            ::

                >>> specifier = rhythmmakertools.DurationSpellingSpecifier()
                >>> print(format(specifier))
                rhythmmakertools.DurationSpellingSpecifier(
                    decrease_durations_monotonically=True,
                    )

            ::

                >>> reversed_specifier = specifier.reverse()
                >>> print(format(reversed_specifier))
                rhythmmakertools.DurationSpellingSpecifier(
                    decrease_durations_monotonically=False,
                    )

        Negates `decrecase_monotonically`.

        Returns new duration spelling specifier.
        '''
        decrease_durations_monotonically = not self.decrease_durations_monotonically
        arguments = {
            'decrease_durations_monotonically': decrease_durations_monotonically,
            'forbidden_written_duration': self.forbidden_written_duration,
            }
        result = type(self)(**arguments)
        return result


# -*- encoding: utf-8 -*-
from abjad.tools.abctools import AbjadObject


class StringContactPoint(AbjadObject):
    r'''String contact point indicator.

    ::

        >>> indicator = indicatortools.StringContactPoint('pont')
        >>> print(format(indicator))
        indicatortools.StringContactPoint(
            contact_point='pont',
            )

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_contact_point',
        )

    ### INITIALIZER ###

    def __init__(self,
        contact_point=None,
        ):
        self._contact_point = contact_point

    ### PUBLIC PROPERTIES ###

    @property
    def contact_point(self):
        r'''Gets contact point.

        ::

            >>> indicator = indicatortools.StringContactPoint('tasto')
            >>> indicator.contact_point
            'tasto'

        '''
        return self._contact_point



# -*- encoding: utf-8 -*-


def cumulative_products(sequence):
    r'''Cumulative products of `sequence`.

    ::

        >>> mathtools.cumulative_products([1, 2, 3, 4, 5, 6, 7, 8])
        [1, 2, 6, 24, 120, 720, 5040, 40320]

    ::

        >>> mathtools.cumulative_products([1, -2, 3, -4, 5, -6, 7, -8])
        [1, -2, -6, 24, 120, -720, -5040, 40320]

    Raises type error when `sequence` is neither list nor tuple.

    Raises value error on empty `sequence`.

    Returns list.
    '''

    if not isinstance(sequence, (list, tuple)):
        raise TypeError

    if len(sequence) == 0:
        raise ValueError

    result = [sequence[0]]
    for element in sequence[1:]:
        result.append(result[-1] * element)

    return result


# -*- encoding: utf-8 -*-
import re
from abjad.tools.stringtools.is_snake_case \
	import underscore_delimited_lowercase_regex_body


underscore_delimited_lowercase_package_regex_body = """
    (%s\.)*
    %s
    """ % (
        underscore_delimited_lowercase_regex_body, 
        underscore_delimited_lowercase_regex_body
        )

underscore_delimited_lowercase_package_regex = re.compile(
    '^%s$' %
    underscore_delimited_lowercase_package_regex_body, 
    re.VERBOSE
    )

def is_snake_case_package_name(expr):
    r'''Is true when `expr` is a string and is underscore-delimited lowercase 
    package name.

    ..  container:: example

        ::

            >>> stringtools.is_snake_case_package_name('foo.bar.blah_package')
            True

    Otherwise false:

    ..  container:: example

        ::

            >>> stringtools.is_snake_case_package_name('foo.bar.BlahPackage')
            False

    Returns boolean.
    '''

    if not isinstance(expr, str):
        return False

    return bool(underscore_delimited_lowercase_package_regex.match(expr))


# -*- encoding: utf-8 -*-
from __future__ import print_function
import importlib
import os
import shutil
from abjad.tools import abctools


class AbjadAPIGenerator(abctools.AbjadObject):
    r'''Generates theAbjad API restructed text.

        * writes ReST pages for individual classes and functions
        * writes the API index ReST
        * handles sorting tools packages into composition, manual-loading
          and unstable
        * handles ignoring private tools packages

    '''

    ### CLASS VARIABLES ###

    _api_title = 'Abjad API'

    _package_descriptions = {
        'core': 'Core composition packages',
        'demos': 'Demos and example packages',
        'internals': 'Abjad internal packages',
        'score manager': 'Abjad IDE packages',
        'unstable': 'Unstable packages (load manually)',
        }

    _undocumented_packages = (
        'materialpackages',
        'scorepackages',
        )

    ### INITIALIZER ###

    def __init__(self):
        pass

    ### SPECIAL METHODS ###

    def __call__(self, verbose=False):
        r'''Calls Abjad API generator.

        Returns none.
        '''
        from abjad.tools import documentationtools

        if verbose:
            print('Now writing restructured text files ...')
            print('')

        ignored_directory_names = [
            '__pycache__',
            '.git',
            '.svn',
            'scores',
            'test',
            ]
        ignored_directory_names.extend(self._undocumented_packages)

        api_index_document = documentationtools.ReSTDocument()
        api_index_document.append(documentationtools.ReSTHeading(
            level=0,
            text=self._api_title,
            ))

        documentation_sections = {}
        for code_path, docs_path, package_prefix in self.path_definitions:
            if not os.path.exists(code_path):
                os.makedirs(code_path)
            if not os.path.exists(docs_path):
                os.makedirs(docs_path)
            self._prune_obsolete_documents(code_path, docs_path)
            for name in os.listdir(code_path):
                if name in ignored_directory_names:
                    continue
                path = os.path.join(code_path, name)
                if not os.path.isdir(path):
                    continue
                if not os.path.exists(os.path.join(
                    path, '__init__.py')):
                    continue
                packagesystem_path = ''.join((package_prefix, name))
                module = importlib.import_module(packagesystem_path)
                documenter = documentationtools.ToolsPackageDocumenter(
                    module,
                    ignored_directory_names=ignored_directory_names,
                    prefix=package_prefix,
                    )
                if not documenter.all_documenters:
                    continue
                section = documenter.documentation_section
                if section not in documentation_sections:
                    documentation_sections[section] = []
                payload = (documenter, code_path, docs_path, package_prefix)
                documentation_sections[section].append(payload)

        for section in sorted(documentation_sections):
            documenters = sorted(
                documentation_sections[section],
                key=lambda x: x[0].module_name,
                )
            text = self._package_descriptions.get(
                section,
                'Undefined documentation section',
                )
            section_heading = documentationtools.ReSTHeading(
                level=1,
                text=text,
                )
            api_index_document.append(section_heading)
            for payload in documenters:
                tools_package_documenter = payload[0]
                code_path = payload[1]
                docs_path = payload[2]
                package_prefix = payload[3]
                tools_package_toc = \
                    tools_package_documenter.create_api_toc_section()
                api_index_document.extend(tools_package_toc)
                self._write_document(
                    tools_package_documenter,
                    code_path,
                    docs_path,
                    package_prefix,
                    )
                for documenter in tools_package_documenter.all_documenters:
                    self._write_document(
                        documenter,
                        code_path,
                        docs_path,
                        package_prefix,
                        )

        documentationtools.Documenter.write(
            self.docs_api_index_path,
            api_index_document.rest_format,
            )

        if verbose:
            #print ''
            print('... done.')
            print('')

    ### PRIVATE METHODS ###

    def _prune_obsolete_documents(self, code_path, docs_path):
        for directory, directory_names, file_names in os.walk(docs_path):
            path_suffix = os.path.relpath(directory, docs_path)
            for file_name in file_names:
                if not file_name.endswith('.rst') or file_name == 'index.rst':
                    continue
                code_file_path = os.path.join(
                    code_path,
                    path_suffix,
                    file_name.replace('.rst', '.py'),
                    )
                if not os.path.exists(code_file_path):
                    docs_file_path = os.path.join(
                        directory,
                        file_name,
                        )
                    os.remove(docs_file_path)
                    print('PRUNING', os.path.relpath(docs_file_path))
            for directory_name in directory_names[:]:
                code_directory = os.path.join(
                    code_path,
                    path_suffix,
                    directory_name,
                    )
                should_delete = False
                if not os.path.exists(code_directory):
                    should_delete = True
                elif not [x for x in os.listdir(code_directory)
                    if x.endswith('.py')]:
                    should_delete = True
                # sphinx seems to want a docs/source/_static directory
                if directory_name.startswith('_'):
                    should_delete = False
                    directory_names.remove(directory_name)
                if should_delete:
                    docs_directory = os.path.join(
                        directory,
                        directory_name,
                        )
                    shutil.rmtree(docs_directory)
                    print('PRUNING', os.path.relpath(docs_directory))

    def _write_document(self, documenter, code_path, docs_path, package_prefix):
        from abjad.tools import documentationtools
        parts = documenter.module_name.replace(
            package_prefix, '', 1).split('.')
        if isinstance(documenter, documentationtools.ToolsPackageDocumenter):
            parts.append('index.rst')
        elif isinstance(documenter, (documentationtools.ClassDocumenter,
            documentationtools.FunctionDocumenter)):
            parts.pop()
            parts[-1] += '.rst'
        parts.insert(0, docs_path)
        file_path = os.path.join(*parts)
        directory = os.path.dirname(file_path)
        if not os.path.exists(directory):
            os.makedirs(directory)
        documenter.write(file_path, documenter())

    ### PUBLIC PROPERTIES ###

    @property
    def docs_api_index_path(self):
        r'''Path to index.rst for Abjad API.
        '''
        from abjad import abjad_configuration
        return os.path.join(
            abjad_configuration.abjad_directory,
            'docs', 'source', 'api', 'index.rst')

    @property
    def package_prefix(self):
        r'''Package prefix.
        '''
        return ('abjad.tools.', 'abjad.demos.')

    @property
    def path_definitions(self):
        r'''Code path / docs path / package prefix triples.
        '''
        from abjad import abjad_configuration
        tools_code_path = os.path.join(
            abjad_configuration.abjad_directory,
            'tools',
            )
        tools_docs_path = os.path.join(
            abjad_configuration.abjad_directory,
            'docs',
            'source',
            'api',
            'tools',
            )
        tools_package_prefix = 'abjad.tools.'
        tools_triple = (
            tools_code_path,
            tools_docs_path,
            tools_package_prefix,
            )
        demos_code_path = os.path.join(
            abjad_configuration.abjad_directory,
            'demos',
            )
        demos_docs_path = os.path.join(
            abjad_configuration.abjad_directory,
            'docs',
            'source',
            'api',
            'demos',
            )
        demos_package_prefix = 'abjad.demos.'
        demos_triple = (
            demos_code_path,
            demos_docs_path,
            demos_package_prefix,
            )
        all_triples = (tools_triple, demos_triple)
        return all_triples

    @property
    def root_package(self):
        r'''Root package.

        Returns ``'abjad'``.
        '''
        return 'abjad'

    @property
    def tools_package_path_index(self):
        r'''Tools package path index.

        Returns ``2``.
        '''
        return 2


# -*- encoding: utf-8 -*-
from abjad.tools.selectiontools.Selection import Selection


class SimultaneousSelection(Selection):
    r'''SliceSelection of components taken simultaneously.

    Simultaneously selections implement no duration properties.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        )

    ### PUBLIC METHODS ###

    def get_vertical_moment_at(self, offset):
        r'''Select vertical moment at `offset`.
        '''
        from abjad.tools import selectiontools
        return selectiontools.VerticalMoment(self, offset)


# -*- encoding: utf-8 -*-
from abjad import *


def test_mathtools_all_are_nonnegative_integer_equivalent_numbers_01():

    numbers = [0, 0.0, Fraction(0)]
    assert mathtools.all_are_nonnegative_integer_equivalent_numbers(
        numbers)

    numbers = [2, 3, Fraction(4)]
    assert mathtools.all_are_nonnegative_integer_equivalent_numbers(
        numbers)


def test_mathtools_all_are_nonnegative_integer_equivalent_numbers_02():

    numbers = [-1, -2.0, Fraction(-3)]
    assert not mathtools.all_are_nonnegative_integer_equivalent_numbers(
        numbers)


# -*- encoding: utf-8 -*-
from abjad.tools.abctools.AbjadObject import AbjadObject


class Clef(AbjadObject):
    r'''A clef.

    ::

        >>> clef = Clef('treble')
        >>> clef
        Clef(name='treble')

    ::

        >>> staff = Staff("c'8 d'8 e'8 f'8 g'8 a'8 b'8 c''8")
        >>> show(staff) # doctest: +SKIP

    ::

        >>> clef = Clef('treble')
        >>> attach(clef, staff)
        >>> clef = Clef('alto')
        >>> attach(clef, staff[1])
        >>> clef = Clef('bass')
        >>> attach(clef, staff[2])
        >>> clef = Clef('treble^8')
        >>> attach(clef, staff[3])
        >>> clef = Clef('bass_8')
        >>> attach(clef, staff[4])
        >>> clef = Clef('tenor')
        >>> attach(clef, staff[5])
        >>> clef = Clef('bass^15')
        >>> attach(clef, staff[6])
        >>> clef = Clef('percussion')
        >>> attach(clef, staff[7])
        >>> show(staff) # doctest: +SKIP

    ..  doctest::

        >>> print(format(staff))
        \new Staff {
            \clef "treble"
            c'8
            \clef "alto"
            d'8
            \clef "bass"
            e'8
            \clef "treble^8"
            f'8
            \clef "bass_8"
            g'8
            \clef "tenor"
            a'8
            \clef "bass^15"
            b'8
            \clef "percussion"
            c''8
        }

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_default_scope',
        '_middle_c_position',
        '_name',
        )

    _clef_name_to_middle_c_position = {
        'treble': -6,
        'alto': 0,
        'tenor': 2,
        'bass': 6,
        'french': -8,
        'soprano': -4,
        'mezzosoprano': -2,
        'baritone': 4,
        'varbaritone': 4,
        'percussion': 0,
        'tab': 0
        }

    _format_slot = 'opening'

    ### INITIALIZER ###

    def __init__(self, name='treble'):
        from abjad.tools import scoretools
        self._default_scope = scoretools.Staff
        if isinstance(name, str):
            self._name = name
        elif isinstance(name, type(self)):
            self._name = name.name
        else:
            message = 'can not initialize clef: {!r}.'
            message = message.format(name)
            raise TypeError(message)
        middle_c_position = self._calculate_middle_c_position(self._name)
        self._middle_c_position = middle_c_position

    ### SPECIAL METHODS ###

    def __copy__(self, *args):
        r'''Copies clef.

        ::

            >>> import copy
            >>> clef_1 = Clef('alto')
            >>> clef_2 = copy.copy(clef_1)

        ::

            >>> clef_1, clef_2
            (Clef(name='alto'), Clef(name='alto'))

        ::

            >>> clef_1 == clef_2
            True

        ::

            >>> clef_1 is clef_2
            False

        Returns new clef.
        '''
        return type(self)(self.name)

    def __eq__(self, expr):
        r'''Is true when `expr` is a clef with name equal to that of this clef.
        Otherwise false.

        ::

            >>> clef_1 = Clef('treble')
            >>> clef_2 = Clef('alto')

        ::

            >>> clef_1 == clef_1
            True
            >>> clef_1 == clef_2
            False
            >>> clef_2 == clef_1
            False
            >>> clef_2 == clef_2
            True

        Returns boolean.
        '''
        if isinstance(expr, type(self)):
            return self._name == expr._name
        return False

    def __format__(self, format_specification=''):
        r'''Formats clef.

        Set `format_specification` to `''`, `'lilypond'` or `'storage'`.
        Interprets `''` equal to `'storage'`.

        ::

            >>> clef = Clef('treble')
            >>> print(format(clef))
            indicatortools.Clef(
                name='treble',
                )

        Returns string.
        '''
        superclass = super(Clef, self)
        return superclass.__format__(format_specification=format_specification)

    def __hash__(self):
        r'''Hashes clef.

        Required to be explicitely re-defined on Python 3 if __eq__ changes.

        Returns integer.
        '''
        return super(Clef, self).__hash__()

    def __ne__(self, arg):
        r'''Is true when clef of `arg` does not equal clef name of clef.
        Otherwise false.

        ::

            >>> clef_1 = Clef('treble')
            >>> clef_2 = Clef('alto')

        ::

            >>> clef_1 != clef_1
            False
            >>> clef_1 != clef_2
            True
            >>> clef_2 != clef_1
            True
            >>> clef_2 != clef_2
            False

        Returns boolean.
        '''
        superclass = super(Clef, self)
        return superclass.__ne__(arg)

    ### PRIVATE PROPERTIES ###

    @property
    def _attribute_manifest(self):
        from abjad.tools import systemtools
        from scoremanager import idetools
        return systemtools.AttributeManifest(
            systemtools.AttributeDetail(
                name='name',
                command='nm',
                editor=idetools.getters.get_string,
                ),
            )

    @property
    def _contents_repr_string(self):
        return repr(self._name)

    @property
    def _lilypond_format(self):
        return r'\clef "{}"'.format(self._name)

    ### PRIVATE METHODS ###

    def _calculate_middle_c_position(self, clef_name):
        alteration = 0
        if '_' in self._name:
            base_name, part, suffix = clef_name.partition('_')
            if suffix == '8':
                alteration = 7
            elif suffix == '15':
                alteration = 13
            else:
                message = 'bad clef alteration suffix: {!r}.'
                message = message.format(suffix)
                raise Exception(message)
        elif '^' in self._name:
            base_name, part, suffix = clef_name.partition('^')
            if suffix == '8':
                alteration = -7
            elif suffix == '15':
                alteration = -13
            else:
                message = "bad clef alteration suffix: {!r}."
                message = message.format(suffix)
                raise Exception(message)
        else:
            base_name = clef_name
        return self._clef_name_to_middle_c_position[base_name] + alteration

    @classmethod
    def _list_clef_names(cls):
        return list(sorted(cls._clef_name_to_middle_c_position))

    ### PUBLIC PROPERTIES ###

    @property
    def middle_c_position(self):
        r'''Middle C position of clef.

        ::

            >>> clef = Clef('treble')
            >>> clef.middle_c_position
            -6

        Returns integer number of stafflines.
        '''
        return self._middle_c_position

    @property
    def name(self):
        r'''Name of clef.

        Returns string.
        '''
        return self._name


# -*- encoding: utf-8 -*-
from abjad import *


def test_sequencetools_join_subsequences_by_sign_of_elements_01():

    sequence_1 = [[1, 2], [3, 4], [-5, -6, -7], [-8, -9, -10], [11, 12]]
    sequence_2 = sequencetools.join_subsequences_by_sign_of_elements(sequence_1)

    assert sequence_2 == [[1, 2, 3, 4], [-5, -6, -7, -8, -9, -10], [11, 12]]


def test_sequencetools_join_subsequences_by_sign_of_elements_02():

    sequence_1 = [[1, 2], [], [], [3, 4, 5], [6, 7]]
    sequence_2 = sequencetools.join_subsequences_by_sign_of_elements(sequence_1)

    assert sequence_2 == [[1, 2], [], [3, 4, 5, 6, 7]]


# -*- encoding: utf-8 -*-


def make_n_middle_c_centered_pitches(n):
    '''Make `n` middle-c centered pitches, where 0 < `n`:

    ::

        >>> for p in pitchtools.make_n_middle_c_centered_pitches(5): p
        NamedPitch('f')
        NamedPitch('a')
        NamedPitch("c'")
        NamedPitch("e'")
        NamedPitch("g'")

    ::

        >>> for p in pitchtools.make_n_middle_c_centered_pitches(4): p
        NamedPitch('g')
        NamedPitch('b')
        NamedPitch("d'")
        NamedPitch("f'")

    Returns list of zero or more named pitches.
    '''
    from abjad.tools import pitchtools

    if n == 0:
        return []
    indices = list(range(0, 2 * abs(n), 2))
    if n < 0:
        indices.reverse()
    average = int(sum(indices) / float(abs(n)))
    centered = [x - average for x in indices]
    letters = ['c', 'd', 'e', 'f', 'g', 'a', 'b']
    tups = [divmod(x, 7) for x in centered]
    pitch_names = [
        letters[x[1]] + str(pitchtools.Octave(x[0] + 4))
        for x in tups]
    return [pitchtools.NamedPitch(x) for x in pitch_names]



# -*- encoding: utf-8 -*-
from abjad import *


def test_scoretools_Skip___ne___01():

    skip_1 = scoretools.Skip((1, 4))
    skip_2 = scoretools.Skip((1, 4))
    skip_3 = scoretools.Skip((1, 8))

    assert skip_1 != skip_2
    assert skip_1 != skip_3
    assert skip_2 != skip_3


# -*- encoding: utf-8 -*-
import time
from abjad.tools.abctools import ContextManager


class ForbidUpdate(ContextManager):
    r'''A context manager for forbidding score updates.

    ..  container:: example

        ::

            >>> staff = Staff("c'32 d'2.. ~ d'16 e'32")
            >>> with systemtools.ForbidUpdate(component=staff):
            ...     for x in staff[:]:
            ...         mutate(x).replace(Chord(x))
            ...

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_component',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        component=None,
        ):
        from abjad.tools import scoretools
        prototype = (scoretools.Component, type(None))
        assert isinstance(component, prototype)
        self._component = component

    ### SPECIAL METHODS ###

    def __enter__(self):
        r'''Enters context manager.

        Returns context manager.
        '''
        if self.component is not None:
            self.component._update_now(offsets=True)
            self.component._is_forbidden_to_update = True
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        r'''Exist context manager.

        Returns none.
        '''
        if self.component is not None:
            self.component._is_forbidden_to_update = False

    ### PUBLIC PROPERTIES ###

    @property
    def component(self):
        r'''Component of context manager.

        Return component.
        '''
        return self._component


# -*- encoding: utf-8 -*-
import subprocess
from abjad.tools.datastructuretools import TreeContainer
from abjad.tools.documentationtools.GraphvizObject import GraphvizObject


class GraphvizGraph(TreeContainer, GraphvizObject):
    r'''A Graphviz graph.

    ::

        >>> graph = documentationtools.GraphvizGraph(name='G')

    Create other graphviz objects to insert into the graph:

    ::

        >>> cluster_0 = documentationtools.GraphvizSubgraph(name='0')
        >>> cluster_1 = documentationtools.GraphvizSubgraph(name='1')
        >>> a0 = documentationtools.GraphvizNode(name='a0')
        >>> a1 = documentationtools.GraphvizNode(name='a1')
        >>> a2 = documentationtools.GraphvizNode(name='a2')
        >>> a3 = documentationtools.GraphvizNode(name='a3')
        >>> b0 = documentationtools.GraphvizNode(name='b0')
        >>> b1 = documentationtools.GraphvizNode(name='b1')
        >>> b2 = documentationtools.GraphvizNode(name='b2')
        >>> b3 = documentationtools.GraphvizNode(name='b3')
        >>> start = documentationtools.GraphvizNode(name='start')
        >>> end = documentationtools.GraphvizNode(name='end')

    Group objects together into a tree:

    ::

        >>> graph.extend([cluster_0, cluster_1, start, end])
        >>> cluster_0.extend([a0, a1, a2, a3])
        >>> cluster_1.extend([b0, b1, b2, b3])

    Connect objects together with edges:

    ::

        >>> edge = documentationtools.GraphvizEdge()(start, a0)
        >>> edge = documentationtools.GraphvizEdge()(start, b0)
        >>> edge = documentationtools.GraphvizEdge()(a0, a1)
        >>> edge = documentationtools.GraphvizEdge()(a1, a2)
        >>> edge = documentationtools.GraphvizEdge()(a1, b3)
        >>> edge = documentationtools.GraphvizEdge()(a2, a3)
        >>> edge = documentationtools.GraphvizEdge()(a3, a0)
        >>> edge = documentationtools.GraphvizEdge()(a3, end)
        >>> edge = documentationtools.GraphvizEdge()(b0, b1)
        >>> edge = documentationtools.GraphvizEdge()(b1, b2)
        >>> edge = documentationtools.GraphvizEdge()(b2, b3)
        >>> edge = documentationtools.GraphvizEdge()(b2, a3)
        >>> edge = documentationtools.GraphvizEdge()(b3, end)

    Add attributes to style the objects:

    ::

        >>> cluster_0.attributes['style'] = 'filled'
        >>> cluster_0.attributes['color'] = 'lightgrey'
        >>> cluster_0.attributes['label'] = 'process #1'
        >>> cluster_0.node_attributes['style'] = 'filled'
        >>> cluster_0.node_attributes['color'] = 'white'
        >>> cluster_1.attributes['color'] = 'blue'
        >>> cluster_1.attributes['label'] = 'process #2'
        >>> cluster_1.node_attributes['style'] = ('filled', 'rounded')
        >>> start.attributes['shape'] = 'Mdiamond'
        >>> end.attributes['shape'] = 'Msquare'

    Access the computed graphviz format of the graph:

    ::

        >>> print(graph.graphviz_format)
        digraph G {
            subgraph cluster_0 {
                graph [color=lightgrey,
                    label="process #1",
                    style=filled];
                node [color=white,
                    style=filled];
                a0;
                a1;
                a2;
                a3;
                a0 -> a1;
                a1 -> a2;
                a2 -> a3;
                a3 -> a0;
            }
            subgraph cluster_1 {
                graph [color=blue,
                    label="process #2"];
                node [style="filled, rounded"];
                b0;
                b1;
                b2;
                b3;
                b0 -> b1;
                b1 -> b2;
                b2 -> b3;
            }
            start [shape=Mdiamond];
            end [shape=Msquare];
            a1 -> b3;
            a3 -> end;
            b2 -> a3;
            b3 -> end;
            start -> a0;
            start -> b0;
        }

    View the graph:

    ::

        >>> topleveltools.graph(graph) # doctest: +SKIP

    Graphs can also be created without defining names.  Canonical names
    will be automatically determined for all members whose `name` is None:

    ::

        >>> graph = documentationtools.GraphvizGraph()
        >>> graph.append(documentationtools.GraphvizSubgraph())
        >>> graph[0].append(documentationtools.GraphvizNode())
        >>> graph[0].append(documentationtools.GraphvizNode())
        >>> graph[0].append(documentationtools.GraphvizNode())
        >>> graph[0].append(documentationtools.GraphvizSubgraph())
        >>> graph[0][-1].append(documentationtools.GraphvizNode())
        >>> graph.append(documentationtools.GraphvizNode())
        >>> edge = documentationtools.GraphvizEdge()(graph[0][1], graph[1])
        >>> edge = documentationtools.GraphvizEdge()(
        ...     graph[0][0], graph[0][-1][0])

    ::

        >>> print(graph.graphviz_format)
        digraph Graph {
            subgraph cluster_0 {
                node_0_0;
                node_0_1;
                node_0_2;
                subgraph cluster_0_3 {
                    node_0_3_0;
                }
                node_0_0 -> node_0_3_0;
            }
            node_1;
            node_0_1 -> node_1;
        }

    '''

    ### INITIALIZER ###

    def __init__(
        self,
        attributes=None,
        children=None,
        edge_attributes=None,
        is_digraph=True,
        name=None,
        node_attributes=None
        ):
        TreeContainer.__init__(self, children=children, name=name)
        GraphvizObject.__init__(self, attributes=attributes)
        assert isinstance(edge_attributes, (dict, type(None)))
        assert isinstance(node_attributes, (dict, type(None)))
        self._verify_attributes(edge_attributes, '_edge_attributes')
        self._verify_attributes(node_attributes, '_node_attributes')
        self._is_digraph = bool(is_digraph)

    ### PRIVATE PROPERTIES ###

    @property
    def _node_class(self):
        from abjad.tools import documentationtools
        prototype = (
            documentationtools.GraphvizSubgraph,
            documentationtools.GraphvizNode,
            )
        return prototype

    ### PUBLIC PROPERTIES ###

    @property
    def canonical_name(self):
        r'''Canonical name of Graphviz graph.

        Returns string.
        '''
        if self.name is not None:
            return self.name
        return 'Graph'

    @property
    def edge_attributes(self):
        r'''Edge attributes of Graphviz graph.
        '''
        return self._edge_attributes

    @property
    def graphviz_format(self):
        r'''Graphviz format of Graphviz graph.

        Returns string.
        '''
        from abjad.tools import documentationtools

        edges = set([])
        for node in self.nodes[1:]:
            if isinstance(node, (
                documentationtools.GraphvizField,
                documentationtools.GraphvizGroup,
                )):
                continue
            if isinstance(node, documentationtools.GraphvizSubgraph):
                edges.update(node._edges)
            elif isinstance(node, documentationtools.GraphvizNode):
                edges.update(node.all_edges)

        edge_parents = {}
        for edge in edges:
            last_parent = None
            if isinstance(edge.tail, documentationtools.GraphvizField):
                tail_parentage = list(edge.tail.struct.proper_parentage)
            else:
                tail_parentage = list(edge.tail.proper_parentage)
            if isinstance(edge.head, documentationtools.GraphvizField):
                head_parentage = list(edge.head.struct.proper_parentage)
            else:
                head_parentage = list(edge.head.proper_parentage)
            while len(tail_parentage) and len(head_parentage) and \
                tail_parentage[-1] is head_parentage[-1]:
                last_parent = tail_parentage[-1]
                tail_parentage.pop()
                head_parentage.pop()
            if last_parent is None:
                message = 'last parent can not be none.'
                raise Exception(message)
            if last_parent not in edge_parents:
                edge_parents[last_parent] = []
            edge_parents[last_parent].append(edge)

        def recurse(node, indent=0, prefix='subgraph'):
            indent_one = indent * '\t'
            indent_two = (indent + 1) * '\t'
            result = ['{}{} {} {{'.format(indent_one, prefix,
                self._format_value(node.canonical_name))]
            if len(node.attributes):
                contributions = self._format_attribute_list(node.attributes)
                contributions[0] = 'graph {}'.format(contributions[0])
                result.extend(indent_two + x for x in contributions)
            if len(node.node_attributes):
                contributions = \
                    self._format_attribute_list(node.node_attributes)
                contributions[0] = 'node {}'.format(contributions[0])
                result.extend(indent_two + x for x in contributions)
            if len(node.edge_attributes):
                contributions = \
                    self._format_attribute_list(node.edge_attributes)
                contributions[0] = 'edge {}'.format(contributions[0])
                result.extend(indent_two + x for x in contributions)
            for child in node:
                if isinstance(child, type(self)):
                    result.extend(recurse(child, indent=indent+1))
                else:
                    result.extend(indent_two + x \
                        for x in child._graphviz_format_contributions)
            if node in edge_parents:
                edge_contributions = []
                for edge in sorted(edge_parents[node],
                    key=lambda x: (
                    x.tail.canonical_name, x.head.canonical_name)):
                    edge_contributions.extend(indent_two + x \
                        for x in edge._graphviz_format_contributions)
                result.extend(edge_contributions)
            result.append('{}}}'.format(indent_one))
            return result

        if self.is_digraph:
            return '\n'.join(recurse(self, indent=0, prefix='digraph'))
        return '\n'.join(recurse(self, indent=0, prefix='graph'))

    @property
    def is_digraph(self):
        r'''Is true when Graphviz graph is a digraph. Otherwise false.

        Returns boolean.
        '''
        return self._is_digraph

    @is_digraph.setter
    def is_digraph(self, arg):
        self._is_digraph = bool(arg)

    @property
    def node_attributes(self):
        r'''Node attributes of Graphviz graph.
        '''
        return self._node_attributes

    @property
    def unflattened_graphviz_format(self):
        r'''Unflattened Graphviz format of Graphviz graph.

        Returns list.
        '''
        from abjad.tools import systemtools
        assert systemtools.IOManager.find_executable(
            'unflatten'), 'Cannot find `unflatten` command-line tool.'
        graphviz_format = self.graphviz_format
        process = subprocess.Popen('unflatten -l 4'.split(),
            shell=False,
            stdin=subprocess.PIPE,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE
            )
        result = process.communicate(graphviz_format)[0]
        return result


# -*- encoding: utf-8 -*-
import copy
import types
from abjad.tools.selectiontools.ContiguousSelection import ContiguousSelection


class SliceSelection(ContiguousSelection):
    r'''A time-contiguous selection of components all in the same parent.

    ..  container:: example

        ::

            >>> staff = Staff("c'4 d'4 e'4 f'4")
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> print(format(staff))
            \new Staff {
                c'4
                d'4
                e'4
                f'4
            }

        ::

            >>> staff[:2]
            SliceSelection(Note("c'4"), Note("d'4"))

    '''

    ### INITIALIZER ###

    # TODO: assert all are contiguous components in same parent
    def __init__(self, music=None):
        music = self._coerce_music(music)
        ContiguousSelection.__init__(self, music=music)

    ### PRIVATE METHODS ###

    def _get_parent_and_start_stop_indices(self):
        assert self._all_are_contiguous_components_in_same_parent(self)
        if self:
            first, last = self[0], self[-1]
            parent = first._parent
            if parent is not None:
                first_index = parent.index(first)
                last_index = parent.index(last)
                return parent, first_index, last_index
        return None, None, None

    def _give_music_to_empty_container(self, container):
        r'''Not composer-safe.
        '''
        from abjad.tools import scoretools
        assert self._all_are_contiguous_components_in_same_parent(self)
        assert isinstance(container, scoretools.Container)
        assert not container
        music = []
        for component in self:
            music.extend(getattr(component, '_music', ()))
        container._music.extend(music)
        container[:]._set_parents(container)

    def _give_position_in_parent_to_container(self, container):
        r'''Not composer-safe.
        '''
        from abjad.tools import scoretools
        assert self._all_are_contiguous_components_in_same_parent(self)
        assert isinstance(container, scoretools.Container)
        parent, start, stop = self._get_parent_and_start_stop_indices()
        if parent is not None:
            parent._music.__setitem__(slice(start, start), [container])
            container._set_parent(parent)
            self._set_parents(None)



def delimit_words(string):
    r'''Delimits words in `string`.

    ..  container:: example

        Delimits words::

            >>> stringtools.delimit_words('scale degrees 4 and 5.')
            ['scale', 'degrees', '4', 'and', '5']

    ..  container:: example

        Delimits conjoined words::

            >>> stringtools.delimit_words('scale degrees 4and5.')
            ['scale', 'degrees', '4', 'and', '5']

    ..  container:: example

        Delimits lower camel case::

            >>> stringtools.delimit_words('scaleDegrees4and5.')
            ['scale', 'Degrees', '4', 'and', '5']

    ..  container:: example

        Delimits upper camel case::

            >>> stringtools.delimit_words('ScaleDegrees4and 5.')
            ['Scale', 'Degrees', '4', 'and', '5']
        
    ..  container:: example

        Delimits dash case::

            >>> stringtools.delimit_words('scale-degrees-4-and-5.')
            ['scale', 'degrees', '4', 'and', '5']

    ..  container:: example

        Delimits shout case::

            >>> stringtools.delimit_words('SCALE_DEGREES_4_AND_5.')
            ['SCALE', 'DEGREES', '4', 'AND', '5']

    Returns list.
    '''

    assert isinstance(string, str), repr(string)
    words = []
    current_word = ''
    for character in string:
        if not character.isalpha() and not character.isdigit():
            if current_word:
                words.append(current_word)
                current_word = ''
        elif not current_word:
            current_word = current_word + character
        elif character.isupper():
            if current_word[-1].isupper():
                current_word = current_word + character
            else:
                words.append(current_word)
                current_word = character
        elif character.islower():
            if current_word[-1].isalpha():
                current_word = current_word + character
            else:
                words.append(current_word)
                current_word = character
        elif character.isdigit():
            if current_word[-1].isdigit():
                current_word = current_word + character
            else:
                words.append(current_word)
                current_word = character
    if current_word:
        words.append(current_word)
    return words


# -*- encoding: utf-8 -*-
from abjad import *


def test_mathtools_NonreducedFraction_with_multiple_of_denominator_01():

    duration = Duration(1, 2)
    assert mathtools.NonreducedFraction(duration).with_multiple_of_denominator(2) == mathtools.NonreducedFraction(1, 2)
    assert mathtools.NonreducedFraction(duration).with_multiple_of_denominator(4) == mathtools.NonreducedFraction(2, 4)
    assert mathtools.NonreducedFraction(duration).with_multiple_of_denominator(8) == mathtools.NonreducedFraction(4, 8)
    assert mathtools.NonreducedFraction(duration).with_multiple_of_denominator(16) == mathtools.NonreducedFraction(8, 16)


def test_mathtools_NonreducedFraction_with_multiple_of_denominator_02():

    duration = Duration(1, 2)
    assert mathtools.NonreducedFraction(duration).with_multiple_of_denominator(3) == mathtools.NonreducedFraction(3, 6)
    assert mathtools.NonreducedFraction(duration).with_multiple_of_denominator(6) == mathtools.NonreducedFraction(3, 6)
    assert mathtools.NonreducedFraction(duration).with_multiple_of_denominator(12) == mathtools.NonreducedFraction(6, 12)
    assert mathtools.NonreducedFraction(duration).with_multiple_of_denominator(24) == mathtools.NonreducedFraction(12, 24)


def test_mathtools_NonreducedFraction_with_multiple_of_denominator_03():

    duration = Duration(1, 2)
    assert mathtools.NonreducedFraction(duration).with_multiple_of_denominator(5) == mathtools.NonreducedFraction(5, 10)
    assert mathtools.NonreducedFraction(duration).with_multiple_of_denominator(10) == mathtools.NonreducedFraction(5, 10)
    assert mathtools.NonreducedFraction(duration).with_multiple_of_denominator(20) == mathtools.NonreducedFraction(10, 20)
    assert mathtools.NonreducedFraction(duration).with_multiple_of_denominator(40) == mathtools.NonreducedFraction(20, 40)


# -*- encoding: utf-8 -*-
import collections
from abjad.tools import datastructuretools
from abjad.tools.abctools.AbjadObject import AbjadObject


class Performer(AbjadObject):
    r'''A performer.

    ::

        >>> performer = instrumenttools.Performer(name='flutist')
        >>> performer.instruments.append(instrumenttools.Flute())
        >>> performer.instruments.append(instrumenttools.Piccolo())

    ::

        >>> print(format(performer))
        instrumenttools.Performer(
            name='flutist',
            instruments=instrumenttools.InstrumentInventory(
                [
                    instrumenttools.Flute(
                        instrument_name='flute',
                        short_instrument_name='fl.',
                        instrument_name_markup=markuptools.Markup(
                            contents=('Flute',),
                            ),
                        short_instrument_name_markup=markuptools.Markup(
                            contents=('Fl.',),
                            ),
                        allowable_clefs=indicatortools.ClefInventory(
                            [
                                indicatortools.Clef(
                                    name='treble',
                                    ),
                                ]
                            ),
                        pitch_range=pitchtools.PitchRange(
                            range_string='[C4, D7]',
                            ),
                        sounding_pitch_of_written_middle_c=pitchtools.NamedPitch("c'"),
                        ),
                    instrumenttools.Piccolo(
                        instrument_name='piccolo',
                        short_instrument_name='picc.',
                        instrument_name_markup=markuptools.Markup(
                            contents=('Piccolo',),
                            ),
                        short_instrument_name_markup=markuptools.Markup(
                            contents=('Picc.',),
                            ),
                        allowable_clefs=indicatortools.ClefInventory(
                            [
                                indicatortools.Clef(
                                    name='treble',
                                    ),
                                ]
                            ),
                        pitch_range=pitchtools.PitchRange(
                            range_string='[D5, C8]',
                            ),
                        sounding_pitch_of_written_middle_c=pitchtools.NamedPitch("c''"),
                        ),
                    ]
                ),
            )

    The purpose of the class is to model things like
    flute I doubling piccolo and flute.

    At present the class comprises an instrument inventory and name.
    '''

    ### INITIALIZER ###

    def __init__(self, name=None, instruments=None):
        from abjad.tools import instrumenttools
        self._instruments = instrumenttools.InstrumentInventory()
        self.name = name
        self.instruments = instruments

    ### SPECIAL METHODS ###

    def __eq__(self, expr):
        r'''Is true when `expr` is a performer with name and instruments equal to
        those of this performer. Otherwise false.

        Returns boolean.
        '''
        if isinstance(expr, type(self)):
            if self.name == expr.name:
                if self.instruments == expr.instruments:
                    return True
        return False

    def __format__(self, format_specification=''):
        r'''Formats performer.

        Set `format_specification` to `''` or `'storage'`.
        Interprets `''` equal to `'storage'`.

        Returns string.
        '''
        from abjad.tools import systemtools
        if format_specification in ('', 'storage'):
            return systemtools.StorageFormatManager.get_storage_format(self)
        return str(self)

    def __hash__(self):
        r'''Hashes performer.

        Returns string.
        '''
        return hash((type(self).__name__, self.name, tuple(self.instruments)))

    ### PRIVATE PROPERTIES ###

    @property
    def _attribute_manifest(self):
        from abjad.tools import instrumenttools
        from abjad.tools import systemtools
        from scoremanager import idetools
        return systemtools.AttributeManifest(
            systemtools.AttributeDetail(
                name='name',
                command='nm',
                editor=idetools.getters.get_string,
                ),
            systemtools.AttributeDetail(
                name='instruments',
                command='i',
                editor=instrumenttools.InstrumentInventory,
                ),
            )

    @property
    def _one_line_menu_summary(self):
        if not self.instruments:
            result = '{}: no instruments'.format(self.name)
        elif len(self.instruments) == 1 and self.name == \
            self.instruments[0].instrument_name:
            result = '{}'.format(self.name)
        else:
            instruments = ([x.instrument_name for x in self.instruments])
            instruments = ', '.join(instruments)
            result = '{}: {}'.format(self.name, instruments)
        return result

    ### PUBLIC METHODS ###

    def get_instrument(self, instrument_name):
        r'''Gets instrument in performer with `instrument_name`.

        ..  container:: example

            ::
            
                >>> flutist = instrumenttools.Performer(name='flutist')
                >>> flutist.instruments.append(instrumenttools.Flute())
                >>> flutist.instruments.append(instrumenttools.Piccolo())
                >>> flutist.get_instrument('piccolo')
                Piccolo()

        Returns instrument or none.
        '''
        for instrument in self.instruments:
            if instrument.instrument_name == instrument_name:
                return instrument

    @staticmethod
    def list_performer_names():
        r'''Lists performer names.

        ::

            >>> for name in instrumenttools.Performer.list_performer_names():
            ...     name
            ...
            'accordionist'
            'alto'
            'baritone'
            'bass'
            'bassist'
            'bassoonist'
            'cellist'
            'clarinetist'
            'flutist'
            'guitarist'
            'harpist'
            'harpsichordist'
            'hornist'
            'mezzo-soprano'
            'oboist'
            'percussionist'
            'pianist'
            'saxophonist'
            'soprano'
            'tenor'
            'trombonist'
            'trumpeter'
            'tubist'
            'vibraphonist'
            'violinist'
            'violist'
            'xylophonist'

        Returns list.
        '''
        from abjad.tools import instrumenttools
        performer_names = set([])
        for instrument_class in instrumenttools.Instrument._list_instruments():
            instrument = instrument_class()
            performer_name = instrument._get_default_performer_name()
            performer_names.add(performer_name)
        return list(sorted(performer_names))

    @staticmethod
    def list_primary_performer_names():
        r'''List primary performer names.

        ::

            >>> for pair in instrumenttools.Performer.list_primary_performer_names():
            ...     pair
            ...
            ('accordionist', 'acc.')
            ('alto', 'alto')
            ('baritone', 'bar.')
            ('bass', 'bass')
            ('bassist', 'vb.')
            ('bassoonist', 'bsn.')
            ('cellist', 'vc.')
            ('clarinetist', 'cl.')
            ('flutist', 'fl.')
            ('guitarist', 'gt.')
            ('harpist', 'hp.')
            ('harpsichordist', 'hpschd.')
            ('hornist', 'hn.')
            ('mezzo-soprano', 'ms.')
            ('oboist', 'ob.')
            ('pianist', 'pf.')
            ('saxophonist', 'alt. sax.')
            ('soprano', 'sop.')
            ('tenor', 'ten.')
            ('trombonist', 'ten. trb.')
            ('trumpeter', 'tp.')
            ('tubist', 'tb.')
            ('violinist', 'vn.')
            ('violist', 'va.')

        Returns list.
        '''
        from abjad.tools import instrumenttools
        performer_names = set([])
        for instrument_class in instrumenttools.Instrument._list_instruments():
            instrument = instrument_class()
            if instrument._is_primary_instrument:
                performer_name = instrument._get_default_performer_name()
                performer_abbreviation = getattr(
                    instrument, 'performer_abbreviation', None)
                performer_abbreviation = performer_abbreviation or \
                    instrument.short_instrument_name
                performer_names.add((performer_name, performer_abbreviation))
        return list(sorted(performer_names))
    ### PUBLIC PROPERTIES ###

    @property
    def instrument_count(self):
        r'''Number of instruments to be played by performer.

            >>> performer.instrument_count
            2

        Returns nonnegative integer
        '''
        return len(self.instruments)

    @property
    def instruments(self):
        r'''Gets and sets instruments to be played by performer.

        ::

            >>> for instrument in performer.instruments:
            ...     instrument
            Flute()
            Piccolo()

        Returns instrument inventory.
        '''
        return self._instruments

    @instruments.setter
    def instruments(self, instruments):
        from abjad.tools.instrumenttools.Instrument import Instrument
        if instruments is None:
            self._instruments[:] = []
        elif isinstance(instruments,
            (list, datastructuretools.TypedList)):
            assert all(isinstance(x, Instrument) for x in instruments)
            self._instruments[:] = instruments[:]
        else:
            message = 'instruments {!r} must be list or none.'
            message = message.format(instruments)
            raise TypeError(message)

    @property
    def is_doubling(self):
        r'''Is true when performer is to play more than one instrument. Otherwise
        false.

        ::
            >>> performer.is_doubling
            True

        Returns boolean.
        '''
        return 1 < self.instrument_count

    @property
    def likely_instruments_based_on_performer_name(self):
        r'''Likely instruments based on performer name.

        ::

            >>> for likely_instrument in \
            ...     performer.likely_instruments_based_on_performer_name:
            ...     likely_instrument.__name__
            ...
            'AltoFlute'
            'BassFlute'
            'ContrabassFlute'
            'Flute'
            'Piccolo'

        Returns list.
        '''
        dictionary = self.make_performer_name_instrument_dictionary()
        try:
            result = dictionary[self.name]
        except KeyError:
            result = []
        return result

    @property
    def most_likely_instrument_based_on_performer_name(self):
        r'''Most likely instrument based on performer name.

        ::

            >>> performer.most_likely_instrument_based_on_performer_name
            <class 'abjad.tools.instrumenttools.Flute.Flute'>

        Returns instrument class.
        '''
        for likely_instrument_class in \
            self.likely_instruments_based_on_performer_name:
            likely_instrument = likely_instrument_class()
            if likely_instrument._is_primary_instrument:
                return likely_instrument_class

    @property
    def name(self):
        r'''Gets and sets score name of performer.

        ::

            >>> performer.name
            'flutist'

        Returns string.
        '''
        return self._name

    @name.setter
    def name(self, name):
        assert isinstance(name, (str, type(None)))
        self._name = name

    ### PUBLIC METHODS ###

    def make_performer_name_instrument_dictionary(self):
        r'''Makes performer name / instrument dictionary.

        ::

            >>> dictionary = \
            ...     performer.make_performer_name_instrument_dictionary()
            >>> for key, value in sorted(dictionary.items()):
            ...     print(key + ':')
            ...     for x in value:
            ...         print('\t{}'.format(x.__name__))
            accordionist:
                Accordion
            alto:
                AltoVoice
            baritone:
                BaritoneVoice
            bass:
                BassVoice
            bassist:
                Contrabass
            bassoonist:
                Bassoon
                Contrabassoon
            brass player:
                AltoTrombone
                BassTrombone
                FrenchHorn
                TenorTrombone
                Trumpet
                Tuba
            cellist:
                Cello
            clarinetist:
                BassClarinet
                ClarinetInA
                ClarinetInBFlat
                ClarinetInEFlat
                ContrabassClarinet
            clarinettist:
                BassClarinet
                ClarinetInA
                ClarinetInBFlat
                ClarinetInEFlat
                ContrabassClarinet
            contrabassist:
                Contrabass
            double reed player:
                Bassoon
                Contrabassoon
                EnglishHorn
                Oboe
            flautist:
                AltoFlute
                BassFlute
                ContrabassFlute
                Flute
                Piccolo
            flutist:
                AltoFlute
                BassFlute
                ContrabassFlute
                Flute
                Piccolo
            guitarist:
                Guitar
            harpist:
                Harp
            harpsichordist:
                Harpsichord
            hornist:
                FrenchHorn
            instrumentalist:
                Accordion
                AltoFlute
                AltoSaxophone
                AltoTrombone
                AltoVoice
                BaritoneSaxophone
                BaritoneVoice
                BassClarinet
                BassFlute
                Bassoon
                BassSaxophone
                BassTrombone
                BassVoice
                Cello
                ClarinetInA
                ClarinetInBFlat
                ClarinetInEFlat
                Contrabass
                ContrabassClarinet
                ContrabassFlute
                Contrabassoon
                ContrabassSaxophone
                EnglishHorn
                Flute
                FrenchHorn
                Glockenspiel
                Guitar
                Harp
                Harpsichord
                Marimba
                MezzoSopranoVoice
                Oboe
                Piano
                Piccolo
                SopraninoSaxophone
                SopranoSaxophone
                SopranoVoice
                TenorSaxophone
                TenorTrombone
                TenorVoice
                Trumpet
                Tuba
                UntunedPercussion
                Vibraphone
                Viola
                Violin
                Xylophone
            keyboardist:
                Accordion
                Harpsichord
                Piano
            mezzo-soprano:
                MezzoSopranoVoice
            oboist:
                EnglishHorn
                Oboe
            percussionist:
                Glockenspiel
                Marimba
                UntunedPercussion
                Vibraphone
                Xylophone
            pianist:
                Piano
            reed player:
                AltoSaxophone
                BaritoneSaxophone
                BassClarinet
                Bassoon
                BassSaxophone
                ClarinetInA
                ClarinetInBFlat
                ClarinetInEFlat
                ContrabassClarinet
                Contrabassoon
                ContrabassSaxophone
                EnglishHorn
                Oboe
                SopraninoSaxophone
                SopranoSaxophone
                TenorSaxophone
            saxophonist:
                AltoSaxophone
                BaritoneSaxophone
                BassSaxophone
                ContrabassSaxophone
                SopraninoSaxophone
                SopranoSaxophone
                TenorSaxophone
            single reed player:
                AltoSaxophone
                BaritoneSaxophone
                BassClarinet
                BassSaxophone
                ClarinetInA
                ClarinetInBFlat
                ClarinetInEFlat
                ContrabassClarinet
                ContrabassSaxophone
                SopraninoSaxophone
                SopranoSaxophone
                TenorSaxophone
            soprano:
                SopranoVoice
            string player:
                Cello
                Contrabass
                Guitar
                Harp
                Viola
                Violin
            tenor:
                TenorVoice
            trombonist:
                AltoTrombone
                BassTrombone
                TenorTrombone
            trumpeter:
                Trumpet
            tubist:
                Tuba
            vibraphonist:
                Vibraphone
            violinist:
                Violin
            violist:
                Viola
            vocalist:
                AltoVoice
                BaritoneVoice
                BassVoice
                MezzoSopranoVoice
                SopranoVoice
                TenorVoice
            wind player:
                AltoFlute
                AltoSaxophone
                BaritoneSaxophone
                BassClarinet
                BassFlute
                Bassoon
                BassSaxophone
                ClarinetInA
                ClarinetInBFlat
                ClarinetInEFlat
                ContrabassClarinet
                ContrabassFlute
                Contrabassoon
                ContrabassSaxophone
                EnglishHorn
                Flute
                FrenchHorn
                Oboe
                Piccolo
                SopraninoSaxophone
                SopranoSaxophone
                TenorSaxophone
            xylophonist:
                Xylophone



        Returns ordered dictionary.
        '''
        from abjad.tools import instrumenttools
        result = collections.OrderedDict()
        for instrument_class in instrumenttools.Instrument._list_instruments():
            instrument = instrument_class()
            for performer_name in instrument._get_performer_names():
                if performer_name in result:
                    result[performer_name].append(instrument_class)
                else:
                    result[performer_name] = [instrument_class]
        for instruments in result.values():
            instruments.sort(key=lambda x: x.__name__.lower())
        return result


# -*- encoding: utf-8 -*-
from abjad.tools.spannertools.Hairpin import Hairpin


class Crescendo(Hairpin):
    r'''A crescendo.

    ..  container:: example

        ::

            >>> staff = Staff("r4 c'8 d'8 e'8 f'8 r4")
            >>> crescendo = spannertools.Crescendo()
            >>> attach(crescendo, staff[:])
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> print(format(staff))
            \new Staff {
                r4
                c'8 \<
                d'8
                e'8
                f'8 \!
                r4
            }

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        )

    ### INITIALIZER ###

    def __init__(
        self,
        direction=None,
        include_rests=False,
        overrides=None,
        ):
        Hairpin.__init__(
            self,
            descriptor='<',
            direction=direction,
            include_rests=include_rests,
            overrides=overrides,
            )

    ### PUBLIC PROPERTIES ###

    @property
    def descriptor(self):
        r'''Gets descriptor of crescendo.

        ..  container:: example

            ::

                >>> staff = Staff("r4 c'8 d'8 e'8 f'8 r4")
                >>> crescendo = Crescendo()
                >>> attach(crescendo, staff[:])
                >>> show(staff) # doctest: +SKIP

            ::

                >>> crescendo.descriptor
                '<'

        Returns string.
        '''
        return self._descriptor

    @property
    def direction(self):
        r'''Gets direction of crescendo.

        ..  container:: example

            Positions crescendo above staff:

            ::

                >>> staff = Staff("r4 c'8 d'8 e'8 f'8 r4")
                >>> crescendo = Crescendo(direction=Up)
                >>> attach(crescendo, staff[:])
                >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> print(format(staff))
            \new Staff {
                r4
                c'8 ^ \<
                d'8
                e'8
                f'8 \!
                r4
            }

            ::

                >>> crescendo.direction
                '^'

        Returns up, down or none.
        '''
        return self._direction

    @property
    def include_rests(self):
        r'''Gets include-rests flag of crescendo.

        ..  container:: example

            ::

                >>> staff = Staff("r4 c'8 d'8 e'8 f'8 r4")
                >>> crescendo = Crescendo(include_rests=True)
                >>> attach(crescendo, staff[:])
                >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> print(format(staff))
            \new Staff {
                r4 \<
                c'8
                d'8
                e'8
                f'8
                r4 \!
            }

            ::

                >>> crescendo.include_rests
                True

        Returns boolean.
        '''
        return self._include_rests

    @property
    def shape_string(self):
        r'''Gets shape string of crescendo.

        ..  container:: example

            ::

                >>> staff = Staff("r4 c'8 d'8 e'8 f'8 r4")
                >>> crescendo = Crescendo()
                >>> attach(crescendo, staff[:])
                >>> show(staff) # doctest: +SKIP

            ::

                >>> crescendo.shape_string
                '<'

        Returns string.
        '''
        return self._shape_string

    @property
    def start_dynamic(self):
        r'''Gets start dynamic of crescendo.

        ..  container:: example

            ::

                >>> staff = Staff("r4 c'8 d'8 e'8 f'8 r4")
                >>> crescendo = Crescendo()
                >>> attach(crescendo, staff[:])
                >>> show(staff) # doctest: +SKIP

            ::

                >>> crescendo.start_dynamic is None
                True

        Returns dynamic or none.
        '''
        return self._start_dynamic

    @property
    def stop_dynamic(self):
        r'''Gets stop dynamic of crescendo.

        ..  container:: example

            ::

                >>> staff = Staff("r4 c'8 d'8 e'8 f'8 r4")
                >>> crescendo = Crescendo()
                >>> attach(crescendo, staff[:])
                >>> show(staff) # doctest: +SKIP

            ::

                >>> crescendo.stop_dynamic is None
                True

        Returns dynamic or none.
        '''
        return self._stop_dynamic



# -*- encoding: utf-8 -*-
from __future__ import print_function
import abc
import configobj
import os
import time
import validate
from abjad.tools.abctools.AbjadObject import AbjadObject


class Configuration(AbjadObject):
    r'''A configuration object.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_settings',
        )

    ### INITIALIZER ###

    def __init__(self):
        from abjad.tools import systemtools
        # verify configuration directory
        if not os.path.exists(self.configuration_directory):
            os.makedirs(self.configuration_directory)
        # attempt to load config from disk, and validate
        # a config object will be created if none is found on disk
        config = configobj.ConfigObj(
            self.configuration_file_path,
            configspec=self._config_specification
            )
        # validate
        validator = validate.Validator()
        validation = config.validate(validator, copy=True)
        # replace failing key:value pairs with default values
        if validation is not True:
            for key, valid in validation.items():
                if not valid:
                    default = config.default_values[key]
                    message = 'Warning: config key {!r} failed validation,'
                    message += ' setting to default: {!r}.'
                    message = message.format(key, default)
                    print(message)
                    config[key] = default
        # setup output formatting
        config.write_empty_values = True
        config.comments.update(self._option_comments)
        config.initial_comment = self._initial_comment
        # write to disk if doesn't exist
        if not os.path.exists(self.configuration_file_path):
            if not os.path.exists(self.configuration_directory):
                os.makedirs(self.configuration_directory)
            config.write()
        # write to disk if different from current
        else:
            # prevent ConfigObj from automatically writing
            config.filename = None
            with open(self.configuration_file_path, 'r') as f:
                old_config_lines = f.read()

            old_config_lines = old_config_lines.splitlines()
            old_config_lines = [line for line in old_config_lines
                if 'configuration file created on' not in line]
            old_config_lines = '\n'.join(old_config_lines)
            new_config_lines = config.write(None)
            new_config_lines = [line for line in new_config_lines
                if 'configuration file created on' not in line]
            new_config_lines = '\n'.join(new_config_lines)

            lines_are_equal = systemtools.TestManager.compare(
                old_config_lines,
                new_config_lines,
                )
#            print('----------------------------------------')
#            print('TESTING:', type(self))
#            print()
#            print('OLD:')
#            print()
#            print(old_config_lines)
#            print()
#            print('NEW:')
#            print()
#            print(new_config_lines)
#            print()
#            print('EQUAL?', lines_are_equal)
#            print()
            if not lines_are_equal:
#                print('WRITING')
#                print()
                with open(self.configuration_file_path, 'w') as file_pointer:
                    config.write(file_pointer)
        # turn the ConfigObj instance into a standard dict,
        # and replace its empty string values with Nones,
        # caching the result on this AbjadConfiguration instance.
        self._settings = dict(config)
        for key, value in self._settings.items():
            if value == '' or value == 'None':
                self._settings[key] = None

    ### SPECIAL METHODS ###

    def __delitem__(self, i):
        r'''Deletes item `i` from configuration.

        Returns none.
        '''
        del(self._settings[i])

    def __getitem__(self, i):
        r'''Gets item `i` from configuration.

        Returns none.
        '''
        return self._settings[i]

    def __iter__(self):
        r'''Iterates configuration settings.

        Returns generator.
        '''
        for key in self._settings:
            yield key

    def __len__(self):
        r'''Gets the number of settings in configuration.

        Returns nonnegative integer.
        '''
        return len(self._settings)

    def __setitem__(self, i, arg):
        r'''Sets configuration item `i` to `arg`.

        Returns none.
        '''
        self._settings[i] = arg

    ### PRIVATE METHODS ###

    @abc.abstractmethod
    def _get_option_definitions(self):
        raise NotImplementedError

    ### PRIVATE PROPERTIES ###

    @property
    def _config_specification(self):
        specs = self._option_specification
        return ['{} = {}'.format(key, value)
            for key, value in sorted(specs.items())]

    @property
    def _current_time(self):
        return time.strftime("%d %B %Y %H:%M:%S")

    @abc.abstractproperty
    def _initial_comment(self):
        raise NotImplementedError

    @property
    def _option_comments(self):
        options = self._get_option_definitions()
        comments = [(key, options[key]['comment']) for key in options]
        return dict(comments)

    @property
    def _option_specification(self):
        options = self._get_option_definitions()
        specs = [(key, options[key]['spec']) for key in options]
        return dict(specs)

    ### PUBLIC PROPERTIES ###

    @abc.abstractproperty
    def configuration_directory(self):
        r'''Gets configuration directory.

        Returns string.
        '''
        raise NotImplementedError

    @abc.abstractproperty
    def configuration_file_name(self):
        r'''Gets configuration file name.

        Returns string.
        '''
        raise NotImplementedError

    @property
    def configuration_file_path(self):
        r'''Gets configuration file path.

        Returns string.
        '''
        return os.path.join(
            self.configuration_directory,
            self.configuration_file_name,
            )

    @property
    def home_directory(self):
        r'''Gets home directory.

        Returns string.
        '''
        path = os.environ.get('HOME') or \
            os.environ.get('HOMEPATH') or \
            os.environ.get('APPDATA')
        return os.path.abspath(path)


# -*- encoding: utf-8 -*-
import copy
from abjad.tools import durationtools
from abjad.tools import indicatortools
from abjad.tools.abctools import AbjadObject


class PitchArrayRow(AbjadObject):
    r'''A pitch array row.

    ::

        >>> array = pitchtools.PitchArray([[1, 2, 1], [2, 1, 1]])
        >>> array[0].cells[0].pitches.append(0)
        >>> array[0].cells[1].pitches.append(2)
        >>> array[1].cells[2].pitches.append(4)
        >>> print(array)
        [c'] [d'    ] [  ]
        [       ] [ ] [e']

    ::

        >>> array[0]
        PitchArrayRow(c', d' x2, x1)

    ::

        >>> array[0].cell_widths
        (1, 2, 1)

    ::

        >>> array[0].dimensions
        (1, 4)

    ::

        >>> array[0].pitches
        (NamedPitch("c'"), NamedPitch("d'"))

    '''

    ### INITIALIZER ###

    def __init__(self, cells=None):
        from abjad.tools import pitchtools
        self._parent_array = None
        #self._pitch_range = pitchtools.PitchRange(None, None)
        self._pitch_range = pitchtools.PitchRange()
        self._cells = []
        cells = cells or []
        self.extend(cells)

    ### SPECIAL METHODS ###

    def __add__(self, arg):
        r'''Concatenates `arg` to pitch array row.

        Returns new pitch array row.
        '''
        if not isinstance(arg, PitchArrayRow):
            message = 'must be pitch array row.'
            raise TypeError(message)
        self_copy = copy.copy(self)
        arg_copy = copy.copy(arg)
        new_row = PitchArrayRow([])
        new_row.extend(self_copy.cells)
        new_row.extend(arg_copy.cells)
        return new_row

    def __copy__(self):
        r'''Copies pitch array row.

        Returns new pitch array row.
        '''
        new_cells = []
        for cell in self.cells:
            new_cell = copy.copy(cell)
            new_cells.append(new_cell)
        return PitchArrayRow(new_cells)

    def __eq__(self, arg):
        r'''Is true when `arg` is a pitch array row with contents equal to that of
        this pitch array row. Otherwise false.

        Returns boolean.
        '''
        if isinstance(arg, PitchArrayRow):
            for self_cell, arg_cell in zip(self.cells, arg.cells):
                if not self_cell.matches_cell(arg_cell):
                    return False
                return True
        return False

    def __getitem__(self, arg):
        r'''Gets pitch array cell `arg` from pitch array row.

        Returns pitch array cell.
        '''
        if isinstance(arg, int):
            if 0 <= arg < self.width:
                accumulated_width = 0
                for cell in self.cells:
                    total_width = accumulated_width + cell.width
                    if accumulated_width <= arg < total_width:
                        return cell
                    accumulated_width = total_width
            elif 0 < abs(arg) < self.width:
                accumulated_width = 0
                abs_arg = abs(arg)
                for cell in reversed(self.cells):
                    total_width = accumulated_width + cell.width
                    if accumulated_width < abs_arg <= total_width:
                        return cell
                    accumulated_width = total_width
            else:
                message = 'no such cell in row.'
                raise IndexError(message)
        elif isinstance(arg, slice):
            cells = []
            start, stop, step = arg.indices(self.width)
            for cell_index in range(start, stop, step):
                cell = self[cell_index]
                if len(cells) == 0:
                    cells.append(cell)
                else:
                    if cells[-1] is not cell:
                        cells.append(cell)
            cells = tuple(cells)
            return cells
        else:
            message = 'must be int or slice.'
            raise ValueError(message)

    def __getstate__(self):
        r'''Gets state of pitch array row.
        '''
        return vars(self)

    def __hash__(self):
        r'''Hashes pitch array row.

        Required to be explicitely re-defined on Python 3 if __eq__ changes.

        Returns integer.
        '''
        return super(PitchArrayRow, self).__hash__()

    def __iadd__(self, arg):
        r'''Adds `arg` to pitch array row in place.

        Returns pitch array row.
        '''
        if not isinstance(arg, PitchArrayRow):
            message = 'must be pitch array row.'
            raise TypeError(message)
        copy_arg = copy.copy(arg)
        self.extend(copy_arg.cells)
        return self

    def __len__(self):
        r'''Length of pitch array row.

        Defined equal to the width of pitch array row.

        Returns nonnegative integer.
        '''
        return self.width

    def __ne__(self, arg):
        r'''Is true when pitch array row does not equal `arg`. Otherwise false.

        Returns boolean.
        '''
        return not self == arg

    def __repr__(self):
        r'''Gets interpreter representation of pitch array row.

        Returns string.
        '''
        return '{}({})'.format(type(self).__name__, self._compact_summary)

    def __str__(self):
        r'''String representation of pitch array row.

        Returns string.
        '''
        result = [str(cell) for cell in self.cells]
        result = ' '.join(result)
        return result

    ### PRIVATE PROPERTIES ###

    @property
    def _compact_summary(self):
        len_self = len(self.cells)
        if not len_self:
            return ''
        elif 0 < len_self <= 8:
            result = [
                cell._format_row_column_repr_string for cell in self.cells]
            return ', '.join(result)
        else:
            left = ', '.join(
                [x._format_row_column_repr_string for x in self.cells[:2]])
            right = ', '.join(
                [x._format_row_column_repr_string for x in self.cells[-2:]])
            number_in_middle = len_self - 4
            middle = ', ... [%s] ..., '% number_in_middle
            return left + middle + right

    @property
    def _format_contents_string(self):
        result = []
        for cell in self.cells:
            result.append(cell._format_row_column_repr_string)
        result = ', '.join(result)
        return result

    ### PUBLIC PROPERTIES ###

    @property
    def cell_tokens(self):
        r'''Cell items of pitch array row.

        Returns tuple.
        '''
        return tuple([cell.item for cell in self.cells])

    @property
    def cell_widths(self):
        r'''Cell widths of pitch array row.

        Returns tuple.
        '''
        return tuple([cell.width for cell in self.cells])

    @property
    def cells(self):
        r'''Cells of pitch array row.

        Returns tuple.
        '''
        return tuple(self._cells)

    @property
    def depth(self):
        r'''Depth of pitch array row.

        Defined equal to ``1``.

        Returns ``1``.
        '''
        return 1

    @property
    def dimensions(self):
        r'''Dimensions of pitch array row.

        Returns pair.
        '''
        return self.depth, self.width

    @property
    def is_defective(self):
        r'''Is true when width of pitch array row does not equal width of parent
        pitch array. Otherwise false.

        Returns boolean.
        '''
        if self.parent_array is not None:
            return not self.width == self.parent_array.width
        return False

    @property
    def is_in_range(self):
        r'''Is true when all pitches in pitch array row are in pitch range of
        pitch array row. Otherwise false.

        Returns boolean.
        '''
        return all(pitch in self.pitch_range for pitch in self.pitches)

    @property
    def parent_array(self):
        r'''Parent pitch array housing pitch array row.

        Returns pitch array or none.
        '''
        return self._parent_array

    @property
    def pitch_range(self):
        r'''Gets and set pitch range of pitch array row.

        Returns pitch range.
        '''
        return self._pitch_range

    @pitch_range.setter
    def pitch_range(self, arg):
        from abjad.tools import pitchtools
        if not isinstance(arg, pitchtools.PitchRange):
            message = 'must be pitch range.'
            raise TypeError(message)
        self._pitch_range = arg

    @property
    def pitches(self):
        r'''Pitches in pitch array row.

        Returns tuple.
        '''
        pitches = []
        for cell in self.cells:
            pitches.extend(cell.pitches)
        return tuple(pitches)

    @property
    def row_index(self):
        r'''Row index of pitch array row in parent pitch array.

        Returns nonnegative integer.
        '''
        parent_array = self.parent_array
        if parent_array is not None:
            return parent_array._rows.index(self)
        message = 'row has no parent array.'
        raise IndexError(message)

    @property
    def weight(self):
        r'''Weight of pitch array row.

        Defined equal to sum of weights of pitch array cells in pitch array
        row.

        Returns nonnegative integer.
        '''
        return sum([cell.weight for cell in self.cells])

    @property
    def width(self):
        r'''Width of pitch array row.

        Defined equal to sum of widths of pitch array cells in pitch array row.

        Returns nonnegative integer.
        '''
        return sum([cell.width for cell in self.cells])

    ### PUBLIC METHODS ###

    def append(self, cell_token):
        r'''Appends `cell_token` to pitch array row.

        Returns none.
        '''
        from abjad.tools import pitchtools
        cell = pitchtools.PitchArrayCell(cell_token)
        cell._parent_row = self
        self._cells.append(cell)

    def apply_pitches(self, pitch_tokens):
        r'''Applies `pitch_tokens` to pitch cells in pitch array row.

        Returns none.
        '''
        pitch_tokens = pitch_tokens[:]
        if pitch_tokens:
            for cell in self.cells:
                if cell.pitches:
                    cell.pitches = [pitch_tokens.pop(0)]
        else:
            self.empty_pitches()

    def copy_subrow(self, start=None, stop=None):
        r'''Copies subrow of pitch array row from `start` to `stop`.

        Returns new pitch array row.
        '''
        arg = slice(start, stop)
        start, stop, step = arg.indices(self.width)
        if not step == 1:
            message = 'step no implemented.'
            raise NotImplementedError(message)
        column_indices = set(range(start, stop, step))
        row = PitchArrayRow([])
        cells = self[arg]
        new_cells = []
        for cell in cells:
            if not cell in new_cells:
                trim = [
                    x for x in cell.column_indices if x not in column_indices]
                new_width = cell.width - len(trim)
                new_cell = copy.copy(cell)
                new_cell._width = new_width
                new_cells.append(new_cell)
        row.extend(new_cells)
        return row

    def empty_pitches(self):
        r'''Empties pitches in pitch array row.

        Returns none.
        '''
        for cell in self.cells:
            cell.pitches = []

    def extend(self, cell_tokens):
        r'''Extends `cell_tokens` against pitch array row.

        Returns none.
        '''
        for cell_token in cell_tokens:
            self.append(cell_token)

    def has_spanning_cell_over_index(self, i):
        r'''Is true when pitch array row has one or more cells spanning over
        index `i`. Otherwise false.

        Returns boolean.
        '''
        try:
            cell = self[i]
            return cell.column_indices[0] < i
        except IndexError:
            return False

    def index(self, cell):
        r'''Index of pitch array `cell` in pitch array row.

        Retunrs nonnegative integer.
        '''
        return self._cells.index(cell)

    def merge(self, cells):
        r'''Merges `cells`.

        Returns pitch array cell.
        '''
        from abjad.tools import pitchtools
        column_indices = []
        pitches = []
        width = 0
        for cell in cells:
            if not isinstance(cell, pitchtools.PitchArrayCell):
                raise TypeError
            if not cell.parent_row is self:
                message = 'cells must belong to row.'
                raise ValueError(message)
            column_indices.extend(cell.column_indices)
            pitches.extend(cell.pitches)
            width += cell.width
        start = min(column_indices)
        stop = start + len(column_indices)
        strict_series = list(range(start, stop))
        if not column_indices == strict_series:
            message = 'cells must be contiguous.'
            raise ValueError(message)
        first_cell = cells[0]
        for cell in cells[1:]:
            self.remove(cell)
        first_cell._pitches = pitches
        first_cell._width = width
        return first_cell

    def pad_to_width(self, width):
        r'''Pads pitch array row to `width`.

        Returns none.
        '''
        from abjad.tools import pitchtools
        self_width = self.width
        if width < self_width:
            message = 'pad width must not be less than row width.'
            raise ValueError(message)
        missing_width = width - self_width
        for i in range(missing_width):
            cell = pitchtools.PitchArrayCell()
            self.append(cell)

    def pop(self, cell_index):
        r'''Pops cell `cell_index` from pitch array row.

        Returns pitch array cell.
        '''
        cell = self.pop(cell_index)
        cell._parent_row = None
        return cell

    def remove(self, cell):
        r'''Removes `cell` form pitch array row.

        Returns none.
        '''
        for i, x in enumerate(self.cells):
            if x is cell:
                self._cells.pop(i)
                break
        cell._parent_row = None

    def to_measure(self, cell_duration_denominator=8):
        r'''Changes pitch array row to measure with time signature
        numerator equal to pitch array row width and
        time signature denominator equal to `cell_duration_denominator`.

        ::

            >>> array = pitchtools.PitchArray([
            ...     [1, (2, 1), ([-2, -1.5], 2)],
            ...     [(7, 2), (6, 1), 1]])

        ::

            >>> print(array)
            [  ] [d'] [bf bqf    ]
            [g'     ] [fs'   ] [ ]

        ::

            >>> measure = array.rows[0].to_measure()

        ..  doctest::

            >>> print(format(measure))
            {
                \time 4/8
                r8
                d'8
                <bf bqf>4
            }

        Returns measure.
        '''
        from abjad.tools import scoretools
        time_signature = indicatortools.TimeSignature(
            (self.width, cell_duration_denominator))
        measure = scoretools.Measure(time_signature, [])
        basic_cell_duration = \
            durationtools.Duration(1, cell_duration_denominator)
        measure_pitches, measure_durations = [], []
        for cell in self.cells:
            cell_pitches = cell.pitches
            if not cell_pitches:
                measure_pitches.append(None)
            elif len(cell_pitches) == 1:
                measure_pitches.append(cell_pitches[0])
            else:
                measure_pitches.append(cell_pitches)
            measure_duration = cell.width * basic_cell_duration
            measure_durations.append(measure_duration)
        leaves = scoretools.make_leaves(measure_pitches, measure_durations)
        measure.extend(leaves)
        return measure

    def withdraw(self):
        r'''Withdraws pitch array row from parent pitch array.

        Returns pitch array row.
        '''
        if self.parent_array is not None:
            self.parent_array.remove_row(self)
        return self


# -*- encoding: utf-8 -*-
from abjad import *
from abjad.tools import sievetools


def test_sievetools_ResidueClass___init___01():
    r'''Initialize from modulo and residue.
    '''

    rc = sievetools.ResidueClass(6, 0)

    assert isinstance(rc, sievetools.ResidueClass)
    assert rc.modulo == 6
    assert rc.residue == 0


def test_sievetools_ResidueClass___init___02():
    r'''Initialize from other rc instance.
    '''

    rc = sievetools.ResidueClass(sievetools.ResidueClass(6, 0))

    assert isinstance(rc, sievetools.ResidueClass)
    assert rc.modulo == 6
    assert rc.residue == 0


# -*- encoding: utf-8 -*-
import pytest
from abjad import *


def test_sequencetools_sum_elements_01():
    r'''Sum slices cyclically at every fourth index.
    '''

    sequence_1 = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11]

    sequence_2 = sequencetools.sum_elements(sequence_1, [(0, 1)], period=4)
    assert sequence_2 == [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11]

    sequence_2 = sequencetools.sum_elements(sequence_1, [(0, 2)], period=4)
    assert sequence_2 == [1, 2, 3, 9, 6, 7, 17, 10, 11]

    sequence_2 = sequencetools.sum_elements(sequence_1, [(0, 3)], period=4)
    assert sequence_2 == [3, 3, 15, 7, 27, 11]

    sequence_2 = sequencetools.sum_elements(sequence_1, [(0, 4)], period=4)
    assert sequence_2 == [6, 22, 38]


def test_sequencetools_sum_elements_02():
    r'''Sum slice at only the zeroth index.
    Append overhang elements.
    '''

    sequence_1 = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11]

    sequence_2 = sequencetools.sum_elements(sequence_1, [(0, 1)])
    assert sequence_2 == [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11]

    sequence_2 = sequencetools.sum_elements(sequence_1, [(0, 2)])
    assert sequence_2 == [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11]

    sequence_2 = sequencetools.sum_elements(sequence_1, [(0, 3)])
    assert sequence_2 == [3, 3, 4, 5, 6, 7, 8, 9, 10, 11]

    sequence_2 = sequencetools.sum_elements(sequence_1, [(0, 4)])
    assert sequence_2 == [6, 4, 5, 6, 7, 8, 9, 10, 11]


def test_sequencetools_sum_elements_03():
    r'''Sum every 5, 6, 7 or 8 elements together.
    Do append incomplete final sums.
    '''

    sequence_1 = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11]

    sequence_2 = sequencetools.sum_elements(sequence_1, [(0, 5)], period=5, overhang=True)
    assert sequence_2 == [10, 35, 21]

    sequence_2 = sequencetools.sum_elements(sequence_1, [(0, 6)], period=6, overhang=True)
    assert sequence_2 == [15, 51]

    sequence_2 = sequencetools.sum_elements(sequence_1, [(0, 7)], period=7, overhang=True)
    assert sequence_2 == [21, 45]

    sequence_2 = sequencetools.sum_elements(sequence_1, [(0, 8)], period=8, overhang=True)
    assert sequence_2 == [28, 38]


def test_sequencetools_sum_elements_04():
    r'''Sum every 5, 6, 7 or 8 elements together.
    Do not append incomplete final sums.
    '''

    sequence_1 = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11]

    sequence_2 = sequencetools.sum_elements(sequence_1, [(0, 5)], period=5, overhang=False)
    assert sequence_2 == [10, 35]

    sequence_2 = sequencetools.sum_elements(sequence_1, [(0, 6)], period=6, overhang=False)
    assert sequence_2 == [15, 51]

    sequence_2 = sequencetools.sum_elements(sequence_1, [(0, 7)], period=7, overhang=False)
    assert sequence_2 == [21]

    sequence_2 = sequencetools.sum_elements(sequence_1, [(0, 8)], period=8, overhang=False)
    assert sequence_2 == [28]


def test_sequencetools_sum_elements_05():
    r'''Sum at multiple points in each period.
    '''

    sequence_1 = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11]

    sequence_2 = sequencetools.sum_elements(sequence_1, [(0, 2), (2, 2)], period=5)
    assert sequence_2 == [1, 5, 4, 11, 15, 9, 21]


def test_sequencetools_sum_elements_06():
    r'''Affected indices must be less than period.
    '''

    sequence_1 = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11]
    statement = 'sequencetools.sum_elements(sequence_1, [(0, 99)], period=4)'
    assert pytest.raises(ValueError, statement)


# -*- encoding: utf-8 -*-
from abjad import *


def test_pitchtools_OctaveTranspositionMappingInventory_append_01():
    r'''Append named item.
    '''

    inventory = pitchtools.OctaveTranspositionMappingInventory()
    assert repr(inventory) == 'OctaveTranspositionMappingInventory([])'


# -*- encoding: utf-8 -*-
from abjad import *


def test_pitchtools_sort_named_pitch_carriers_in_expr_01():
    r'''Works on notes.
    '''

    chord = Chord([-12, -10, -2, 4, 8, 11, 17, 19, 27, 30, 33, 37], (1, 4))
    sorted_pitches = pitchtools.sort_named_pitch_carriers_in_expr(
        chord.written_pitches)

    sorted_pitch_numbers = [
        pitchtools.NumberedPitch(pitch).pitch_number
        for pitch in sorted_pitches
        ]
    sorted_pcs = [
        pitchtools.NumberedPitchClass(pitch)
        for pitch in sorted_pitches
        ]

    assert sorted_pitch_numbers == [
        -12, 37, -10, 27, 4, 17, 30, 19, 8, 33, -2, 11]
    assert sorted_pcs == [
        pitchtools.NumberedPitchClass(n) for n in range(12)]



# -*- encoding: utf-8 -*-
from abjad.tools import mathtools


def yield_all_k_ary_sequences_of_length(k, length):
    '''Yields all `k`-ary sequences of `length`.

    ::

        >>> for sequence in sequencetools.yield_all_k_ary_sequences_of_length(2, 3):
        ...     sequence
        ...
        (0, 0, 0)
        (0, 0, 1)
        (0, 1, 0)
        (0, 1, 1)
        (1, 0, 0)
        (1, 0, 1)
        (1, 1, 0)
        (1, 1, 1)

    Returns generator of tuples.
    '''

    assert isinstance(k, int) and 1 <= k
    assert isinstance(length, int) and 0 <= length

    total_sequences = k ** length
    for rank in range(total_sequences):
        result = list(mathtools.integer_to_base_k_tuple(rank, k))
        n_leading_zeros = length - len(result)
        leading_zeros = n_leading_zeros * [0]
        # to avoid pychecker slice assignment bug
        #result[0:0] = leading_zeros
        result.__setitem__(slice(0, 0), leading_zeros)
        yield tuple(result)


# -*- encoding: utf-8 -*-
from abjad import *


def test_labeltools_label_leaves_in_expr_with_named_interval_classes_01():

    pitch_numbers = [0, 25, 11, -4, -14, -13, 9, 10, 6, 5]
    staff = Staff(scoretools.make_notes(pitch_numbers, [Duration(1, 8)]))
    labeltools.label_leaves_in_expr_with_named_interval_classes(staff)

    assert systemtools.TestManager.compare(
        staff,
        r"""
        \new Staff {
            c'8 ^ \markup { +aug8 }
            cs'''8 ^ \markup { -M2 }
            b'8 ^ \markup { -aug2 }
            af8 ^ \markup { -m7 }
            bf,8 ^ \markup { aug1 }
            b,8 ^ \markup { +m7 }
            a'8 ^ \markup { +m2 }
            bf'8 ^ \markup { -dim4 }
            fs'8 ^ \markup { aug1 }
            f'8
        }
        """
        )

    assert inspect_(staff).is_well_formed()


# -*- encoding: utf-8 -*-
from abjad import *


def test_mathtools_all_are_pairs_01():

    assert mathtools.all_are_pairs([(1, 2), (3, 4), (5, 6), (7, 8)])
    assert mathtools.all_are_pairs([])


def test_mathtools_all_are_pairs_02():

    assert not mathtools.all_are_pairs('foo')
    assert not mathtools.all_are_pairs(1.5)
    assert not mathtools.all_are_pairs([1, 2])
    assert not mathtools.all_are_pairs([(1, 2), (3, 4), (5, 6, 7)])


# -*- encoding: utf-8 -*-
from abjad.tools.datastructuretools.TypedList import TypedList
from abjad.tools.pitchtools.OctaveTranspositionMapping \
    import OctaveTranspositionMapping


class OctaveTranspositionMappingInventory(TypedList):
    '''An ordered list of octave transposition mappings.

    ::

        >>> mapping_1 = pitchtools.OctaveTranspositionMapping(
        ...     [('[A0, C4)', 15), ('[C4, C8)', 27)])
        >>> mapping_2 = pitchtools.OctaveTranspositionMapping(
        ...     [('[A0, C8]', -18)])
        >>> inventory = pitchtools.OctaveTranspositionMappingInventory(
        ...     [mapping_1, mapping_2])

    ::

        >>> print(format(inventory))
        pitchtools.OctaveTranspositionMappingInventory(
            [
                pitchtools.OctaveTranspositionMapping(
                    [
                        pitchtools.OctaveTranspositionMappingComponent(
                            source_pitch_range=pitchtools.PitchRange(
                                range_string='[A0, C4)',
                                ),
                            target_octave_start_pitch=pitchtools.NumberedPitch(15),
                            ),
                        pitchtools.OctaveTranspositionMappingComponent(
                            source_pitch_range=pitchtools.PitchRange(
                                range_string='[C4, C8)',
                                ),
                            target_octave_start_pitch=pitchtools.NumberedPitch(27),
                            ),
                        ]
                    ),
                pitchtools.OctaveTranspositionMapping(
                    [
                        pitchtools.OctaveTranspositionMappingComponent(
                            source_pitch_range=pitchtools.PitchRange(
                                range_string='[A0, C8]',
                                ),
                            target_octave_start_pitch=pitchtools.NumberedPitch(-18),
                            ),
                        ]
                    ),
                ]
            )

    Octave transposition mapping inventories implement list interface
    and are mutable.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        )

    ### PRIVATE PROPERTIES ###

    @property
    def _attribute_manifest(self):
        from abjad.tools import systemtools
        return systemtools.AttributeManifest()

    @property
    def _item_callable(self):
        return OctaveTranspositionMapping